/*
 * Copyright (c) [2012] Samsung Electronics Co., Ltd.
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <app.h>
#include <Elementary.h>

#include "mp-util.h"
#include "mp-video-log.h"
#include "video-player.h"
#include "mp-external-ug.h"
#include "mp-video-value-define.h"
#include "mp-video-string-define.h"
#include "mp-video-service-ctrl.h"


#define UG_LIST_ITEMS_SIZE_MAX			32
#define	UG_SHARE_MENU_ITEM_MAX			6


static ui_gadget_h pUiGadgetHandle = NULL;
static Ecore_Event_Handler *pHardKeyEventHandler = NULL;
static MpExternalUGType nCurUGType = MP_EXTERNAL_UG_NONE;
static char **szUgItems = NULL;


static Evas_Object 	*pUGPopupHandle = NULL;
static Evas_Object 	*pUGGenList = NULL;


typedef struct _External_Ug {
	MpExternalUGType nType;
	char *szUgName;
} MpExternalUG;

MpExternalUG st_MpExternalUG[] = {
	{MP_EXTERNAL_UG_MESSAGE, "msg-composer-efl" },
	{MP_EXTERNAL_UG_EMAIL, "email-composer-efl" },
	{MP_EXTERNAL_UG_MYFILE_DETAIL, "myfile-detail-efl" },
	{MP_EXTERNAL_UG_YOUTUBE, "youtube-efl" },
	{MP_EXTERNAL_UG_BLUETOOTH, "setting-bluetooth-efl" },
	{MP_EXTERNAL_UG_FTM, "fileshare-efl" },
	{MP_EXTERNAL_UG_FACEBOOK, "facebook-efl" },
};




/*
 * CALLBACK FUNCTION
 */

static char *MpExternalUgGetLabelOfGenlistItemCb(const void *pUserData, Evas_Object *pObject, const char *pPart)
{
	VideoLogInfo("");

	int nIndex = (int)pUserData;

	if(!strcmp(pPart, "elm.text"))
	{
		VideoLogInfo("Index : %d", nIndex);
		return strdup(szUgItems[nIndex]);
	}

	return NULL;
}


static Eina_Bool MpExternalUgHardkeyDownCb(void *pUserData, int nType,
					   void *pEvent)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return EINA_FALSE;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	MpExternalUGType nExternalUgState = MpExternalUgGetCurUgState();

	if (nExternalUgState <= MP_EXTERNAL_UG_NONE ||
	    nExternalUgState >= MP_EXTERNAL_UG_MAX) {
		VideoLogInfo
		    ("Return hardkey callback function because of no UG mode.");
		return EINA_FALSE;
	}

	Ecore_Event_Key *pKeyEvent = (Ecore_Event_Key *) pEvent;

	if (!strcmp(pKeyEvent->keyname, KEY_END)) {
		VideoLogInfo("Bakc(End) key");
		MpExternalUgUnload();
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}
	if (!strcmp(pKeyEvent->keyname, HOME_KEY)) {
		VideoLogInfo("Home key");
		MpExternalUgUnload();
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}

	return EINA_FALSE;
}

void MpExternalUgLayoutCb(ui_gadget_h pUiGadget, enum ug_mode nMode, void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	Evas_Object *pLayoutBase = NULL;
	Evas_Object *pWin = NULL;

	pLayoutBase = ug_get_layout(pUiGadget);
	if (!pLayoutBase) {
		ug_destroy(pUiGadget);
		VideoLogError("[ERR]");
		return;
	}
	pWin = ug_get_window();

	switch (nMode) {
	case UG_MODE_FULLVIEW:
		evas_object_size_hint_weight_set(pLayoutBase, EVAS_HINT_EXPAND,
						 EVAS_HINT_EXPAND);
		elm_win_resize_object_add(pWin, pLayoutBase);
		evas_object_show(pLayoutBase);
		break;

	default:
		VideoLogInfo("[DEFAUT!!!!!!!!]");
		break;
	}
}

void MpExternalUgResultCb(ui_gadget_h pUiGadget, service_h pService, void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}
}

void MpExternalUgDestroyCb(ui_gadget_h pUiGadget, void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	MpExternalUgUnload();

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowLayout(pAppData);
}

static void MpExternalUGDestroyPopupList(void)
{
	VideoLogInfo("");
	int nCount = 0;
	if(szUgItems)
	{
		for(nCount = 0; nCount < MP_EXTERNAL_UG_MAX; nCount++)
		{
			if(szUgItems[nCount])
			{
				free(szUgItems[nCount]);
				szUgItems[nCount] = NULL;
			}
		}
		free(szUgItems);
		szUgItems = NULL;
	}

	if(pUGGenList)
	{
		evas_object_del(pUGGenList);
		pUGGenList = NULL;
	}

	if(pUGPopupHandle) {
		evas_object_del(pUGPopupHandle);
		pUGPopupHandle = NULL;
	}

	MpUtilSetAlphaOfMainWindow(TRUE);
}


static void MpExternalUGPopupCb(void *pUserData, Evas_Object *pObject,
				     void *pEventInfo)
{
	VideoLogInfo("");

	if(pUserData == NULL)
	{
		VideoLogInfo("[ERR]No have pUserData");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowLayout(pAppData);

	MpExternalUGDestroyPopupList();
}

static void MpExternalUgResFn(service_h pRequest, service_h pReply, service_result_e nResult, void *pUserData)
{
	if(!pUserData)
{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowLayout(pAppData);
}

static void MpExternalUGAppSvcLoad(MpExternalUGType nExternalUGType, void *pUserData)
{

	VideoLogInfo("");

	if (pUserData == NULL) {
		VideoLogInfo("[ERR]No have pUserData");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	service_h pServiceHandle = NULL;
	int 	ret_val = 0;
	char	pkg_name[STR_LEN_MAX] = {0,};

	if(service_create(&pServiceHandle) != SERVICE_ERROR_NONE)
	{
		VideoLogInfo("[ERR]service_create(&pServiceHandle)");
		return;
	}

	switch(nExternalUGType)
	{
		case MP_EXTERNAL_UG_YOUTUBE:
			strncpy(pkg_name,"org.tizen.youtube",STR_LEN_MAX-1);
			break;

		case MP_EXTERNAL_UG_FACEBOOK:
			strncpy(pkg_name,"org.tizen.facebook",STR_LEN_MAX-1);
			break;
		default:
			break;
	}

	if(service_set_operation(pServiceHandle, SERVICE_OPERATION_SEND) != SERVICE_ERROR_NONE)
	{
		if(pServiceHandle)
		{
			service_destroy(pServiceHandle);
		}

		VideoLogInfo("[ERR]service_set_operation fail.");
		return;
	}

	if(nExternalUGType == MP_EXTERNAL_UG_YOUTUBE)
	{
		char xwin_id_str[12] = { 0, };
		Ecore_X_Window xwin_id = (Ecore_X_Window)MpUtilGetMainWindowXID();
		eina_convert_itoa(xwin_id, xwin_id_str);
		VideoLogInfo("For YoutTube XID : %s", xwin_id_str);
		service_add_extra_data(pServiceHandle, "XWINDOW_ID", xwin_id_str);
	}

	if(service_set_mime(pServiceHandle, "video/*") != SERVICE_ERROR_NONE)
	{
		if(pServiceHandle)
	{
			service_destroy(pServiceHandle);
		}

		VideoLogInfo("[ERR]service_set_mime fail.");
		return;
	}

	if(service_set_uri(pServiceHandle, pAppData->szMediaUri) != SERVICE_ERROR_NONE)
	{
		if(pServiceHandle)
		{
			service_destroy(pServiceHandle);
		}

		VideoLogInfo("[ERR]service_set_uri fail.");
		return;
	}


	if(service_set_package(pServiceHandle, pkg_name) != SERVICE_ERROR_NONE)
	{
		if(pServiceHandle)
	{
			service_destroy(pServiceHandle);
		}

		VideoLogInfo("[ERR]service_set_package fail : %d", ret_val);
		return;
	}

	if(service_send_launch_request(pServiceHandle, MpExternalUgResFn, pUserData) != SERVICE_ERROR_NONE)
	{
		if(pServiceHandle)
		{
			service_destroy(pServiceHandle);
	}

		VideoLogInfo("[ERR]service_send_launch_request fail : %d", ret_val);
		return;
	}

	service_destroy(pServiceHandle);
}

static void MpExternalUGSelectListItem(void *pUserData,Evas_Object *pObject,void *pEventInfo)
{
	VideoLogInfo("");

	if(pUserData == NULL)
	{
		VideoLogInfo("[ERR]No have pUserData");
		return;
	}

	Elm_Object_Item *pItem = (Elm_Object_Item *) pEventInfo;

	if(pItem)
	{
		int nIndex = 0;
		nIndex = (int)elm_object_item_data_get(pItem);

		VideoLogInfo("Select Controlbar item of popup : %d", nIndex);

		if(!strcmp(szUgItems[nIndex], MP_SHARE_MESSAGE))
		{
			MpExternalUgLoad(MP_EXTERNAL_UG_MESSAGE, pUserData);
		}
		else if(!strcmp(szUgItems[nIndex], MP_SHARE_EMAIL))
		{
			MpExternalUgLoad(MP_EXTERNAL_UG_EMAIL, pUserData);
		}
		else if(!strcmp(szUgItems[nIndex], MP_SHARE_BLUETOOTH))
		{
			MpExternalUgLoad(MP_EXTERNAL_UG_BLUETOOTH, pUserData);
		}
		else if(!strcmp(szUgItems[nIndex], MP_SHARE_FTM))
		{
			MpExternalUgLoad(MP_EXTERNAL_UG_FTM, pUserData);
		}
		else if(!strcmp(szUgItems[nIndex], MP_SHARE_FACEBOOK))
		{
			MpExternalUGAppSvcLoad(MP_EXTERNAL_UG_FACEBOOK, pUserData);
		}
		else if(!strcmp(szUgItems[nIndex], MP_SHARE_YOUTUBE))
		{
			MpExternalUGAppSvcLoad(MP_EXTERNAL_UG_YOUTUBE, pUserData);
		}

	}
	VideoAppData *pAppData = (VideoAppData *)pUserData;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause(pAppData);

	MpExternalUGDestroyPopupList();
}

/*
 * External Function
 */
bool MpExternalUgLoad(MpExternalUGType nExternalUGType, void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return FALSE;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	struct ug_cbs cbs = { 0, };
	service_h pService;  //bundle *pBundleHandle = NULL;


	cbs.layout_cb = MpExternalUgLayoutCb;
	cbs.result_cb = MpExternalUgResultCb;
	cbs.destroy_cb = MpExternalUgDestroyCb;
	cbs.priv = (void *)pAppData;

	service_create(&pService); //pBundleHandle = bundle_create();

	UG_INIT_EFL(pAppData->pMainWindow, UG_OPT_INDICATOR_ENABLE);

	if(pUiGadgetHandle)
	{
		MpExternalUgUnload();
	}

	switch (nExternalUGType)
	{
	case MP_EXTERNAL_UG_MESSAGE:
		VideoLogInfo("MP_EXTERNAL_UG_MESSAGE");
		//bundle_add(pBundleHandle, "ATTACHFILE", pAppData->szMediaUri);
		service_add_extra_data(pService, "ATTACHFILE", pAppData->szMediaUri);
		break;

	case MP_EXTERNAL_UG_EMAIL:
		VideoLogInfo("MP_EXTERNAL_UG_EMAIL");
		//bundle_add(pBundleHandle, "RUN_TYPE", "5");
		service_add_extra_data(pService, "RUN_TYPE", "5");
		//bundle_add(pBundleHandle, "ATTACHMENT", pAppData->szMediaUri);
		service_add_extra_data(pService, "ATTACHMENT", pAppData->szMediaUri);
		break;

	case MP_EXTERNAL_UG_MYFILE_DETAIL:
		VideoLogInfo("MP_EXTERNAL_UG_MYFILE_DETAIL");
		//bundle_add(pBundleHandle, "Path", pAppData->szMediaUri);
		service_add_extra_data(pService, "Path", pAppData->szMediaUri);
		break;

	case MP_EXTERNAL_UG_YOUTUBE:
		VideoLogInfo("MP_EXTERNAL_UG_YOUTUBE");
		//bundle_add(pBundleHandle, "feature", "video_upload");
		service_add_extra_data(pService, "feature", "video_upload");
		//bundle_add(pBundleHandle, "video", pAppData->szMediaUri);
		service_add_extra_data(pService, "video", pAppData->szMediaUri);
		break;

	case MP_EXTERNAL_UG_FACEBOOK:
		VideoLogInfo("MP_EXTERNAL_UG_FACEBOOK");
		//bundle_add(pBundleHandle, "feature", "video_upload");
		service_add_extra_data(pService, "feature", "video_upload");
		//bundle_add(pBundleHandle, "video", pAppData->szMediaUri);
		service_add_extra_data(pService, "video", pAppData->szMediaUri);
		break;

	case MP_EXTERNAL_UG_BLUETOOTH:
		VideoLogInfo("MP_EXTERNAL_UG_BLUETOOTH");
		//bundle_add(pBundleHandle, "launch-type", "send");
		service_add_extra_data(pService, "launch-type", "send");
		//bundle_add(pBundleHandle, "filecount", "1");
		service_add_extra_data(pService, "filecount", "1");
		//bundle_add(pBundleHandle, "files", pAppData->szMediaUri);
		service_add_extra_data(pService, "files", pAppData->szMediaUri);
		break;

	case MP_EXTERNAL_UG_FTM:
		VideoLogInfo("MP_EXTERNAL_UG_FTM");
		//bundle_add(pBundleHandle, "filecount", "1");
		service_add_extra_data(pService, "filecount", "1");
		//bundle_add(pBundleHandle, "files", pAppData->szMediaUri);
		service_add_extra_data(pService, "files", pAppData->szMediaUri);
		break;

	default:
		service_destroy(pService); //bundle_free(pBundleHandle);
		return FALSE;
	}

	pUiGadgetHandle = ug_create(NULL, st_MpExternalUG[nExternalUGType].szUgName, UG_MODE_FULLVIEW, pService, &cbs);

	nCurUGType = nExternalUGType;

	service_destroy(pService); //bundle_free(pBundleHandle);

	pHardKeyEventHandler = ecore_event_handler_add(ECORE_EVENT_KEY_DOWN, (void *)MpExternalUgHardkeyDownCb, pUserData);

	/*  !!!!!SET ROTATE STATE!!!!!  */

	return TRUE;
}

void MpExternalUgUnload(void)
{
	VideoLogInfo("");

	MpExternalUGDestroyPopupList();

	if (pHardKeyEventHandler) {
		ecore_event_handler_del(pHardKeyEventHandler);
		pHardKeyEventHandler = NULL;
	}

	if (pUiGadgetHandle) {
		ug_destroy(pUiGadgetHandle);
		pUiGadgetHandle = NULL;
	}

	nCurUGType = MP_EXTERNAL_UG_NONE;
}

void MpExternalUgDestroyAll(void)
{
	VideoLogInfo("");

	MpExternalUGDestroyPopupList();

	ug_destroy_all();
}

int MpExternalUgGetCurUgState()
{
	VideoLogInfo("");

	return (int)nCurUGType;
}

void MpExternalUgSetUgState(MpExternalUGType nExternalUGType)
{
	VideoLogInfo("");

	nCurUGType = nExternalUGType;
}

bool MpExternalUgShow(void *pUserData, void *pParent)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return FALSE;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;
	Evas_Object *pButton = NULL;
	Evas_Object *pBox = NULL;

	MpExternalUGDestroyPopupList();

	MpUtilSetAlphaOfMainWindow(FALSE);

	pUGPopupHandle = elm_popup_add(pAppData->pMainWindow);
	elm_object_style_set(pUGPopupHandle, "menustyle");
	elm_object_part_text_set(pUGPopupHandle, "title,text", MP_COM_BUTTON_SHARE);
	evas_object_size_hint_weight_set(pUGPopupHandle, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	pButton = elm_button_add(pUGPopupHandle);
	elm_object_style_set (pButton, "popup_button/default");
	elm_object_text_set(pButton, MP_COM_CLOSE);
	elm_object_part_content_set(pUGPopupHandle, "button1", pButton);
	evas_object_smart_callback_add(pButton, "clicked", MpExternalUGPopupCb, (void*)pAppData);

	Elm_Genlist_Item_Class *st_Share_Itc = NULL;

	st_Share_Itc = elm_genlist_item_class_new();

	st_Share_Itc->item_style = "1text";
	st_Share_Itc->func.text_get = (void *)MpExternalUgGetLabelOfGenlistItemCb;
	st_Share_Itc->func.content_get = NULL;
	st_Share_Itc->func.state_get = NULL;
	st_Share_Itc->func.del = NULL;
	/* Create genlist handle */
	pUGGenList = elm_genlist_add(pUGPopupHandle);
	evas_object_size_hint_weight_set(pUGGenList, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(pUGGenList, EVAS_HINT_FILL, EVAS_HINT_FILL);

	szUgItems = (char **)calloc(1, sizeof(char *) * MP_EXTERNAL_UG_MAX);
	int idx = 0;
	for(idx = 0; idx < MP_EXTERNAL_UG_MAX; idx++)
	{
		szUgItems[idx] = (char *)calloc(1, STR_LEN_MAX * sizeof(char));
	}

	/* Set Sound alive string */
	strncpy(szUgItems[MP_EXTERNAL_UG_MESSAGE], MP_SHARE_MESSAGE, STR_LEN_MAX - 1);
	strncpy(szUgItems[MP_EXTERNAL_UG_EMAIL], MP_SHARE_EMAIL, STR_LEN_MAX - 1);
	strncpy(szUgItems[MP_EXTERNAL_UG_BLUETOOTH], MP_SHARE_BLUETOOTH, STR_LEN_MAX - 1);
	strncpy(szUgItems[MP_EXTERNAL_UG_FTM], MP_SHARE_FTM, STR_LEN_MAX - 1);
	strncpy(szUgItems[MP_EXTERNAL_UG_YOUTUBE], MP_SHARE_YOUTUBE, STR_LEN_MAX - 1);
	strncpy(szUgItems[MP_EXTERNAL_UG_FACEBOOK], MP_SHARE_FACEBOOK, STR_LEN_MAX - 1);

	elm_genlist_item_append(pUGGenList, st_Share_Itc, (void *)MP_EXTERNAL_UG_MESSAGE, NULL, ELM_GENLIST_ITEM_NONE, MpExternalUGSelectListItem, pAppData);
	elm_genlist_item_append(pUGGenList, st_Share_Itc, (void *)MP_EXTERNAL_UG_EMAIL, NULL, ELM_GENLIST_ITEM_NONE, MpExternalUGSelectListItem, pAppData);
	elm_genlist_item_append(pUGGenList, st_Share_Itc, (void *)MP_EXTERNAL_UG_BLUETOOTH, NULL, ELM_GENLIST_ITEM_NONE, MpExternalUGSelectListItem, pAppData);

	elm_genlist_item_class_free(st_Share_Itc);

	pBox = elm_box_add(pUGPopupHandle);
	evas_object_size_hint_min_set(pBox, 614 * elm_config_scale_get(), 338 * elm_config_scale_get());

	elm_box_pack_end(pBox, pUGGenList);
	elm_object_content_set(pUGPopupHandle, pBox);


	evas_object_show(pUGGenList);
	evas_object_show(pUGPopupHandle);


	return TRUE;
}
void MpExternalUgHide()
{
	MpExternalUGDestroyPopupList();
}


