/*
 * Copyright (c) [2012] Samsung Electronics Co., Ltd.
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <app.h>
#include <player.h>
#include <mm_util_imgp.h>
#include <image_util.h>

#include "mp-util.h"
#include "mp-video-log.h"
#include "mp-video-app-ctrl.h"
#include "mp-video-view-mgr.h"
#include "mp-util-media-service.h"
#include "mp-video-value-define.h"
#include "mp-video-string-define.h"
#include "mp-video-streaming-ctrl.h"
#include "mp-external-ug.h"
#include "mp-video-ctrl-mgr.h"
#include "mp-video-service-ctrl.h"

static int nPlayerErrorType = 0;
static int nPlayerInterruptType = 0;
static int nBufferingRate = 0;
static bool bWindowForegroundState = FALSE;

static MpPlayerInterrupteType	g_nCurrentInterrupt = MP_PLAYER_INTERRUPTED_COMPLETED;

static Ecore_Pipe	*pEcorePipeHandleForErrorMsg = NULL;
static Ecore_Pipe	*pEcorePipeHandleBOF = NULL;
static Ecore_Pipe	*pEcorePipeHandleEOF = NULL;
static Ecore_Pipe 	*pEcorePipeHandleForBuffering = NULL;
static Ecore_Pipe 	*pEcorePipeHandleForResumed = NULL;
static Ecore_Pipe 	*pEcorePipeHandleForInterruped = NULL;
static Ecore_Pipe	*pEcorePipeHandlerForSeekComplete = NULL;


static void MpVideoAppCtrlStart(VideoAppData *pAppData);
static void MpVideoAppCtrlStop(VideoAppData *pAppData);
static void MpVideoAppCtrlPause(VideoAppData *pAppData);
static void MpVideoAppCtrlResume(VideoAppData *pAppData);
static void MpVideoAppCtrlStopByAppPause(VideoAppData *pAppData);
static void MpVideoAppCtrlStartByAppResume(VideoAppData *pAppData);
static void MpVideoAppCtrlFFDown(VideoAppData *pAppData);
static void MpVideoAppCtrlRewDown(VideoAppData *pAppData);
static void MpVideoAppCtrlFFUp(VideoAppData *pAppData);
static void MpVideoAppCtrlRewUp(VideoAppData *pAppData);
static void MpVideoAppCtrlSpeedReset(VideoAppData *pAppData);
static void MpVideoAppCtrlSpeedFF(VideoAppData *pAppData);
static void MpVideoAppCtrlSpeedRew(VideoAppData *pAppData);
static void MpVideoAppCtrlRotateScreen(VideoAppData *pAppData, int nRotateState);
static void MpVideoAppCtrlGetPosition(VideoAppData *pAppData);
static void MpVideoAppCtrlSetPosition(VideoAppData *pAppData);
static void MpVideoAppCtrlHideLayout(VideoAppData *pAppData);
static void MpVideoAppCtrlShowLayout(VideoAppData *pAppData);
static void MpVideoAppCtrlHideFinish(VideoAppData *pAppData);
static void MpVideoAppCtrlShowFinish(VideoAppData *pAppData);
static void MpVideoAppCtrlGetVideoResolution(VideoAppData *pAppData);
static void MpVideoAppCtrlReqDurationTime(VideoAppData *pAppData);
static void MpVideoAppCtrlBeginOfStream(VideoAppData *pAppData);
static void MpVideoAppCtrlEndOfStream(VideoAppData *pAppData);
static void MpVideoAppCtrlExit(VideoAppData *pAppData);
static void MpVideoAppCtrlStartStreaming(VideoAppData *pAppData);
static void MpVideoAppCtrlReLaunching(VideoAppData *pAppData);
static void MpVideoAppCtrlGetBufferingPosition(VideoAppData *pAppData);
static void MpVideoAppCtrlPipeInit(VideoAppData *pAppData);
static void MpVideoAppCtrlShowVolume(VideoAppData *pAppData);
static void MpVideoAppCtrlHideVolume(VideoAppData *pAppData);
static void MpVideoAppCtrlSetMute(VideoAppData *pAppData, bool bMute);


/*
 * Callback function
 */

static void MpVideoAppCtrlUnSupportFormatErrorCb(void *pUserData, Evas_Object *pObject, void *pEventInfo)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR] No exist pUserData");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	MpUtilLcdNormalPowerUnlockForPause();

	MpVideoAppCtrlEndOfStream(pAppData);
}


static void MpVideoAppCtrlExitCb(void *pUserData, Evas_Object *pObject, void *pEventInfo)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR] No exist pUserData");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
}

static int MpVideoAppCtrlPauseByInterrupteCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR] No exist pUserData");
		return 0;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	MpVideoAppCtrlPause(pAppData);

	g_nCurrentInterrupt = nPlayerInterruptType;

	switch(nPlayerInterruptType)
	{
	case MP_PLAYER_INTERRUPTED_COMPLETED:
		VideoLogInfo("MP_PLAYER_INTERRUPTED_COMPLETED");
		break;

	case MP_PLAYER_INTERRUPTED_BY_OTHER_APP:
		VideoLogInfo("MP_PLAYER_INTERRUPTED_BY_OTHER_APP");
		break;

	case MP_PLAYER_INTERRUPTED_BY_CALL:
		VideoLogInfo("MP_PLAYER_INTERRUPTED_BY_CALL");
		break;

	case MP_PLAYER_INTERRUPTED_BY_EARJACK_UNPLUG:
		VideoLogInfo("MP_PLAYER_INTERRUPTED_BY_EARJACK_UNPLUG");
		break;

	case MP_PLAYER_INTERRUPTED_BY_RESOURCE_CONFLICT:
		VideoLogInfo("MP_PLAYER_INTERRUPTED_BY_RESOURCE_CONFLICT");
		break;

	case MP_PLAYER_INTERRUPTED_BY_ALARM:
		VideoLogInfo("MP_PLAYER_INTERRUPTED_BY_ALARM");
		break;

	default:
		VideoLogInfo("OTHER");
		break;
	}

	return 0;
}

static int MpVideoAppCtrlBeginOfPlayCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR] No exist pUserData");
		return 0;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerTypeWithoutLog();
	if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoCtrlMgrMediaPlay((void *)pAppData);

		pAppData->bStartSeek = FALSE;

		if(pAppData->nSetPosition > 0)
		{
			VideoLogInfo("Start position time : %dms", pAppData->nSetPosition);
			MpVideoAppCtrlSetPosition(pAppData);
		}

	}
	else
	{
		pAppData->bStartSeek = FALSE;
		if(pAppData->nSetPosition > 0)
		{
			VideoLogInfo("Pending seek position time : %dms", pAppData->nSetPosition);
			MpVideoAppCtrlSetPosition(pAppData);
			pAppData->bStartSeek = TRUE;
		}

		if(MpVideoViewCtrlIsPauseByUser((void *)pAppData))
		{
			MpVideoCtrlMgrSetMute(TRUE);
		}

		if(!pAppData->bStartSeek)
		{
			MpVideoCtrlMgrMediaPlay((void *)pAppData);
		}
	}

	MpVideoAppCtrlBeginOfStream(pAppData);
	MpUtilLcdNormalPowerLockForResume();

	return 0;
}

static int MpVideoAppCtrlEndOfPlayCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR] No exist pUserData");
		return 0;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	MpVideoViewCtrlPause(pAppData);
	MpVideoAppCtrlStop(pAppData);
	MpVideoCtrlMgrMediaDestroy(pAppData);

	MpUtilLcdNormalPowerUnlockForPause();

	MpVideoAppCtrlEndOfStream(pAppData);

	return 0;
}

static int MpVideoAppCtrlBufferingCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist pUserData");
		return 0;
	}

	if(nBufferingRate < 10)
	{
		MpVideoAppCtrlShowLayout((VideoAppData *)pUserData);
	}

	MpVideoViewSetBufferingRateForStreamingView(nBufferingRate, pUserData);

	return 0;
}

static int MpVideoAppCtrlSeekCompletePipeCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR] No exist pUserData");
		return 0;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(pAppData->bStartSeek)
	{
		MpVideoCtrlMgrMediaPlay((void *)pAppData);
		MpVideoAppCtrlBeginOfStream(pAppData);
		MpUtilLcdNormalPowerLockForResume();
		pAppData->bStartSeek = FALSE;
	}

	pAppData->bSeekComplete = TRUE;

	if(pAppData->nSeekPosition > 0) {
		pAppData->nSetPosition = pAppData->nSeekPosition;
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSetPosition(pAppData);
		pAppData->nSeekPosition = 0;
		MpVideoViewCtrlUpdateProgressbar((void *)pAppData);
		return 0;
	}

	if(pAppData->nSeekPlayerState == MP_PLAYER_STATE_PAUSE) {
		VideoLogInfo("");
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause(pAppData);
		pAppData->nSeekPlayerState = MP_PLAYER_STATE_WAIT_FOR_RESPONSE;
	}
	else if(pAppData->nSeekPlayerState == MP_PLAYER_STATE_RESUME) {
		VideoLogInfo("");
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlResume(pAppData);
		pAppData->nSeekPlayerState = MP_PLAYER_STATE_WAIT_FOR_RESPONSE;
	}

	return 0;
}

static void MpVideoAppCtrlSeekCompleteCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist pUserData");
		return;
	}

	VideoLogInfo("");
	VideoAppData *pAppData = (VideoAppData *)pUserData;
	if(pEcorePipeHandlerForSeekComplete)
	{
		if(!ecore_pipe_write(pEcorePipeHandlerForSeekComplete, (void *)pAppData, sizeof(VideoAppData)))
		{
			VideoLogDebug("Fail to write MpVideoAppCtrlSeekCompleteCb.");
		}
	}
}


static int MpVideoAppCtrlResumeByRewCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist pUserData");
		return 0;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	MpVideoAppCtrlResume(pAppData);

	return 0;
}

static int MpVideoAppCtrlErrormsgCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist pUserData");
		return 0;
	}

	VideoLogInfo("");

	switch(MpVideoCtrlMgrGetErrorType(nPlayerErrorType))
	{
	case MP_PLAYER_ERROR_NONE:
		VideoLogInfo("MP_PLAYER_ERROR_NONE");
		break;

	case MP_PLAYER_ERROR_OUT_OF_MEMORY:
		VideoLogInfo("MP_PLAYER_ERROR_OUT_OF_MEMORY");
		MpUtilNotifyPopUp((VideoAppData*)pUserData, _(MP_VPL_UNABLE_TO_PLAY_VIDEO), MpVideoAppCtrlExitCb);
		break;

	case MP_PLAYER_ERROR_INVALID_PARAMETER:
		VideoLogInfo("MP_PLAYER_ERROR_INVALID_PARAMETER");
		MpUtilNotifyPopUp((VideoAppData*)pUserData, _(MP_VPL_UNABLE_TO_PLAY_VIDEO), MpVideoAppCtrlExitCb);
		break;

	case MP_PLAYER_ERROR_NO_SUCH_FILE:
		VideoLogInfo("MP_PLAYER_ERROR_NO_SUCH_FILE");
		MpUtilNotifyPopUp((VideoAppData*)pUserData, MP_COM_POP_FILE_NOT_FOUND, MpVideoAppCtrlExitCb);
		break;

	case MP_PLAYER_ERROR_INVALID_OPERATION:
		VideoLogInfo("MP_PLAYER_ERROR_INVALID_OPERATION");
		MpUtilNotifyPopUp((VideoAppData*)pUserData, _(MP_VPL_UNABLE_TO_PLAY_VIDEO), MpVideoAppCtrlUnSupportFormatErrorCb);
		break;

	case MP_PLAYER_ERROR_SEEK_FAILED:
		VideoLogInfo("MP_PLAYER_ERROR_SEEK_FAILED");
		break;

	case MP_PLAYER_ERROR_INVALID_STATE:
		VideoLogInfo("MP_PLAYER_ERROR_INVALID_STATE");
		break;

	case MP_PLAYER_ERROR_NOT_SUPPORTED_FILE:
		VideoLogInfo("MP_PLAYER_ERROR_NOT_SUPPORTED_FILE");
		MpUtilNotifyPopUp((VideoAppData*)pUserData, _(MP_VPL_POP_UNSUPPORTED_FILE_TYPE), MpVideoAppCtrlUnSupportFormatErrorCb);
		break;

	case MP_PLAYER_ERROR_INVALID_URI:
		VideoLogInfo("MP_PLAYER_ERROR_INVALID_URI");
		MpUtilNotifyPopUp((VideoAppData*)pUserData, MP_COM_POP_INVALID_URL, MpVideoAppCtrlExitCb);
		break;

	case MP_PLAYER_ERROR_SOUND_POLICY:
		VideoLogInfo("MP_PLAYER_ERROR_SOUND_POLICY");
		break;

	case MP_PLAYER_ERROR_CONNECTION_FAILED:
		VideoLogInfo("MP_PLAYER_ERROR_CONNECTION_FAILED");
		MpUtilNotifyPopUp((VideoAppData*)pUserData, MP_COM_POP_CONNECTION_FAILED, MpVideoAppCtrlExitCb);
		break;

	case MP_PLAYER_ERROR_VIDEO_CAPTURE_FAILED:
		VideoLogInfo("MP_PLAYER_ERROR_VIDEO_CAPTURE_FAILED");
		break;

	default:
		MpUtilNotifyPopUp((VideoAppData*)pUserData, _(MP_VPL_UNABLE_TO_PLAY_VIDEO), MpVideoAppCtrlExitCb);
		break;
	}

	return 0;
}

static void MpVideoAppCtrlPlayerPrepareCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	if(pEcorePipeHandleBOF)
	{
		if(!ecore_pipe_write(pEcorePipeHandleBOF, pUserData, sizeof(VideoAppData)))
		{
			VideoLogDebug("FAIL PIPE WRITE");
		}
	}
}

static void MpVideoAppCtrlPlayerCompletedCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	if(pEcorePipeHandleEOF)
	{
		if(!ecore_pipe_write(pEcorePipeHandleEOF, pUserData, sizeof(VideoAppData)))
		{
			VideoLogDebug("FAIL PIPE WRITE");
		}
	}
}

static void MpVideoAppCtrlPlayerInterruptedCb(player_interrupted_code_e nInterruptCode, void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	nPlayerInterruptType = nInterruptCode;

	if(pEcorePipeHandleForInterruped)
	{
		if(!ecore_pipe_write(pEcorePipeHandleForInterruped, pUserData, sizeof(VideoAppData)))
		{
			VideoLogDebug("FAIL PIPE WRITE");
		}
	}
}

static void MpVideoAppCtrlPlayerErrorCb(int nPlayerErrorCode, void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	nPlayerErrorType = nPlayerErrorCode;

	if(pEcorePipeHandleForErrorMsg)
	{
		if(!ecore_pipe_write(pEcorePipeHandleForErrorMsg, pUserData, sizeof(VideoAppData)))
		{
			VideoLogDebug("FAIL PIPE WRITE");
		}
	}
}

static void MpVideoAppCtrlPlayerBufferingCb(int nPercent, void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("Buffering : %d%% ", nPercent);

	nBufferingRate = nPercent;

	if(pEcorePipeHandleForBuffering)
	{
		if(!ecore_pipe_write(pEcorePipeHandleForBuffering, pUserData, sizeof(VideoAppData)))
		{
			VideoLogDebug("FAIL PIPE WRITE");
		}
	}
}

static void MpVideoAppCtrlPlayerUpdateSubtitleCb(unsigned long nDuration, char* szText, void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");
}

/*
 * Internal function
 */
void MpVideoAppCtrlFuncInit(VideoAppData *pAppData)
{
	VideoLogInfo("");

	if(!pAppData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlStart = (void *)MpVideoAppCtrlStart;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlStop = (void *)MpVideoAppCtrlStop;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause = (void *)MpVideoAppCtrlPause;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlResume = (void *)MpVideoAppCtrlResume;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlStopByAppPause = (void *)MpVideoAppCtrlStopByAppPause;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlStartByAppResume = (void *)MpVideoAppCtrlStartByAppResume;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlFFDown = (void *)MpVideoAppCtrlFFDown;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlRewDown = (void *)MpVideoAppCtrlRewDown;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlFFUp = (void *)MpVideoAppCtrlFFUp;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlRewUp = (void *)MpVideoAppCtrlRewUp;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedFF = (void *)MpVideoAppCtrlSpeedFF;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedRew = (void *)MpVideoAppCtrlSpeedRew;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedReset = (void *)MpVideoAppCtrlSpeedReset;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlRotateScreen = (void *)MpVideoAppCtrlRotateScreen;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlGetPosition = (void *)MpVideoAppCtrlGetPosition;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSetPosition = (void *)MpVideoAppCtrlSetPosition;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideLayout = (void *)MpVideoAppCtrlHideLayout;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowLayout = (void *)MpVideoAppCtrlShowLayout;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideFinish = (void *)MpVideoAppCtrlHideFinish;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowFinish = (void *)MpVideoAppCtrlShowFinish;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlGetVideoResolution = (void *)MpVideoAppCtrlGetVideoResolution;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlReqDurationTime = (void *)MpVideoAppCtrlReqDurationTime;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlBeginOfStream = (void *)MpVideoAppCtrlBeginOfStream;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlEndOfStream = (void *)MpVideoAppCtrlEndOfStream;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit = (void *)MpVideoAppCtrlExit;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlStartStreaming = (void *)MpVideoAppCtrlStartStreaming;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlReLaunching = (void *)MpVideoAppCtrlReLaunching;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlGetBufferingPosition = (void *)MpVideoAppCtrlGetBufferingPosition;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowVolume = (void *)MpVideoAppCtrlShowVolume;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideVolume = (void *)MpVideoAppCtrlHideVolume;
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSetMute = (void *)MpVideoAppCtrlSetMute;
}

static void MpVideoAppCtrlGetBufferingPosition(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerTypeWithoutLog();
	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		VideoLogInfo("Skip Normal Play mode");
		return;
	}

	if(MpUtilCheckRTSPType(pAppData->szMediaUri))
	{
		return;
	}

	pAppData->nBufferingPosition = MpVideoCtrlMgrGetBufferingPosition();
}

static void MpVideoAppCtrlStart(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();
	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		if(!MpUtilCheckLocalFileExist(pAppData->szMediaUri))
		{
			MpUtilNotifyPopUp(pAppData, MP_COM_POP_FILE_NOT_EXIST, MpVideoAppCtrlExitCb);
			return;
		}

		MpUtilGetTitleFromFilepath(pAppData->szMediaUri, pAppData->szVideoTitle, STR_LEN_MAX);
		MpVideoViewCtrlChangeTitle(pAppData);
		MpVideoCtrlMgrMediaCreate(pAppData->szMediaUri,
					(void *)MpUtilGetMainWindowXID(),
					NULL,
					pAppData->szSubtitle,
					(void *)pAppData);

		MpVideoCtrlMgrSetPlayerCallback(MpVideoAppCtrlPlayerCompletedCb,
						MpVideoAppCtrlPlayerInterruptedCb,
						MpVideoAppCtrlPlayerErrorCb,
						MpVideoAppCtrlPlayerBufferingCb,
						MpVideoAppCtrlPlayerUpdateSubtitleCb,
						(void*)pAppData);

		if(!MpVideoCtrlMgrMediaRealizeAsync(MpVideoAppCtrlPlayerPrepareCb, (void *)pAppData))
		{
			MpUtilNotifyPopUp(pAppData, _(MP_VPL_UNABLE_TO_PLAY_VIDEO), MpVideoAppCtrlUnSupportFormatErrorCb);
			return;
		}

		MpVideoAppCtrlRotateScreen(pAppData, MpUtilGetRotationState());

	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		if(!MpVideoStreamingCtrlIsNetworkOn())
		{
			MpUtilNotifyPopUp(pAppData, MP_COM_CONNECTION_FAIL, MpVideoAppCtrlExitCb);
			return;
		}

		if(MpVideoStreamingCtrlInit((void *)pAppData))
		{
			VideoLogInfo("Connect to network.");
			MpVideoAppCtrlStartStreaming((void *)pAppData);
		}
	}
}

static void MpVideoAppCtrlStop(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	pAppData->nCurPlayerState = MP_PLAYER_STATE_STOP;
	MpVideoCtrlMgrMediaStop((void *)pAppData);
}

static void MpVideoAppCtrlPause(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	if(pAppData->nCurPlayerState == MP_PLAYER_STATE_PAUSE)
	{
		VideoLogInfo("Allready Pause video player.");
		return;
	}

	pAppData->nCurPlayerState = MP_PLAYER_STATE_PAUSE;

	MpVideoCtrlMgrMediaPause(pAppData);
	MpVideoViewCtrlPause(pAppData);
	MpVideoAppCtrlShowLayout(pAppData);
	MpUtilLcdNormalPowerUnlockForPause();
}

static void MpVideoAppCtrlResume(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	if(MpVideoViewCtrlIsPauseByUser((void *)pAppData))
	{
		return;
	}

	if(pAppData->nCurPlayerState == MP_PLAYER_STATE_RESUME || pAppData->nCurPlayerState == MP_PLAYER_STATE_PLAY)
	{
		VideoLogInfo("Allready Resume video player.");
		return;
	}

	pAppData->nCurPlayerState = MP_PLAYER_STATE_RESUME;
	MpUtilMultimediaKeyGrab();

	MpVideoAppCtrlGetPosition(pAppData);
	MpVideoCtrlMgrMediaResume(pAppData);
	MpVideoViewCtrlResume(pAppData);
	MpVideoAppCtrlShowLayout(pAppData);
	MpUtilLcdNormalPowerLockForResume();
}

static void MpVideoAppCtrlStopByAppPause(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	if(!MpVideoCtrlMgrIsExistPlayerHandle())
	{
		VideoLogInfo("Not exist Player handle.");
		return;
	}

	VideoLogInfo("");

	MpVideoAppCtrlGetPosition(pAppData);
	pAppData->nSetPosition = pAppData->nCurPosition;
	VideoLogInfo("Current position time : %d", pAppData->nSetPosition);
	MpVideoViewCtrlPause(pAppData);
	MpVideoAppCtrlStop(pAppData);
	MpVideoCtrlMgrMediaDestroy(pAppData);
	MpUtilLcdNormalPowerUnlockForPause();

	MpVideoViewCtrlResetProgressBar((void *)pAppData);

	MpUtilMultimediaKeyUngrab();

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();
	if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoViewSetBufferingRateForStreamingView(0, (void *)pAppData);
	}
}

static void MpVideoAppCtrlStartByAppResume(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	if(bWindowForegroundState == FALSE)
	{
		VideoLogInfo("The Window of the video player is hided.");
	}

	MpVideoAppCtrlStart(pAppData);
	MpVideoViewCtrlResume(pAppData);
	MpVideoAppCtrlShowLayout(pAppData);

	return;
}

static void MpVideoAppCtrlFFDown(VideoAppData *pAppData)
{
	VideoLogInfo("");
}

static void MpVideoAppCtrlRewDown(VideoAppData *pAppData)
{
	VideoLogInfo("");
}

static void MpVideoAppCtrlFFUp(VideoAppData *pAppData)
{
	VideoLogInfo("");
}

static void MpVideoAppCtrlRewUp(VideoAppData *pAppData)
{
	VideoLogInfo("");
}

static void MpVideoAppCtrlRotateScreen(VideoAppData *pAppData, int nRotateState)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	if(VIDEO_ROTATE_UNKNOWN > nRotateState || VIDEO_ROTATE_LANDSCAPE_REVERSE < nRotateState)
	{
		VideoLogInfo("Wrong rotate state. - %d", nRotateState);
		return;
	}

	MpVideoCtrlMgrRotateVideoScreen(nRotateState);
}

static void MpVideoAppCtrlGetPosition(VideoAppData *pAppData)
{
	pAppData->nCurPosition = MpVideoCtrlMgrGetPosition();
}

static void MpVideoAppCtrlSetPosition(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	MpVideoCtrlMgrSetPosition(pAppData->nSetPosition, (void *)MpVideoAppCtrlSeekCompleteCb, (void *)pAppData);
	if(MpUtilCheckRTSPType(pAppData->szMediaUri))
	{
		VideoLogInfo("RTSP SEEK Start");
		nBufferingRate = 0;
		MpVideoAppCtrlBufferingCb((void *)pAppData);
	}
}

static void MpVideoAppCtrlSpeedReset(VideoAppData *pAppData)
{
	VideoLogInfo("");
	MpVideoAppCtrlGetPosition(pAppData);
	MpVideoCtrlMgrSetPosition( pAppData->nCurPosition, (void *)MpVideoAppCtrlSeekCompleteCb, (void *)pAppData);
	MpVideoCtrlMgrResetPlaySpeed((void *)pAppData);
}

static void MpVideoAppCtrlSpeedFF(VideoAppData *pAppData)
{
	VideoLogInfo("");
	MpVideoCtrlMgrSetPlaySpeedUp((void *)pAppData);
}

static void MpVideoAppCtrlSpeedRew(VideoAppData *pAppData)
{
	VideoLogInfo("");
	MpVideoCtrlMgrSetPlaySpeedDown((void *)pAppData);
}

static void MpVideoAppCtrlHideLayout(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	MpVideoViewHideLayout((void *)pAppData);
}

static void MpVideoAppCtrlShowLayout(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	MpVideoViewShowLayout((void *)pAppData);
}

static void MpVideoAppCtrlHideFinish(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	MpVideoViewCtrlHideFinish((void *)pAppData, bWindowForegroundState);

}
static void MpVideoAppCtrlShowFinish(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	MpVideoViewCtrlShowFinish((void *)pAppData);
}

static void MpVideoAppCtrlGetVideoResolution(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	pAppData->nVideoWidthResolution = MpVideoCtrlMgrGetVideoWidthResolution();
	pAppData->nVideoHeightResolution = MpVideoCtrlMgrGetVideoHeightResolution();
}

static void MpVideoAppCtrlReqDurationTime(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	pAppData->nDuration = MpVideoCtrlMgrGetDuration();
}

static void MpVideoAppCtrlBeginOfStream(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	MpUtilMultimediaKeyGrab();

	MpVideoAppCtrlGetVideoResolution(pAppData);
	pAppData->nDuration = MpVideoCtrlMgrGetDuration();

	MpVideoViewCtrlInitAfterBeginOfStream((void*)pAppData);

	if(MpVideoViewCtrlIsPauseByUser((void *)pAppData))
	{
		MpVideoAppCtrlPause(pAppData);
		MpVideoCtrlMgrSetMute(FALSE);
	}

	if(MpVideoCtrlMgrIsOnlyAudio())
	{
		MpVideoViewCtrlShowNocontentImage((void*)pAppData);
	}
	else
	{
		MpVideoViewCtrlHideNocontentImage((void*)pAppData);
	}
}

static void MpVideoAppCtrlEndOfStream(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();
	if(nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY)
	{
		if(pAppData->nCurPosition >= 0)
		{
			MpUtilMediaSvcSetVideoLastPlayedTimeByUri(pAppData->szMediaUri, 0);
		}
		else
		{
			VideoLogInfo("Invalid Plaing time or duration time.");
		}
	}
	MpVideoAppCtrlExit(pAppData);
}

static void MpVideoAppCtrlExit(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	if(pEcorePipeHandleForErrorMsg)
	{
		ecore_pipe_del(pEcorePipeHandleForErrorMsg);
		pEcorePipeHandleForErrorMsg = NULL;
	}

	if(pEcorePipeHandleBOF)
	{
		ecore_pipe_del(pEcorePipeHandleBOF);
		pEcorePipeHandleBOF = NULL;
	}

	if(pEcorePipeHandleEOF)
	{
		ecore_pipe_del(pEcorePipeHandleEOF);
		pEcorePipeHandleEOF = NULL;
	}

	if(pEcorePipeHandleForBuffering)
	{
		ecore_pipe_del(pEcorePipeHandleForBuffering);
		pEcorePipeHandleForBuffering = NULL;
	}

	if(pEcorePipeHandleForResumed)
	{
		ecore_pipe_del(pEcorePipeHandleForResumed);
		pEcorePipeHandleForResumed = NULL;
	}

	if(pEcorePipeHandleForInterruped)
	{
		ecore_pipe_del(pEcorePipeHandleForInterruped);
		pEcorePipeHandleForInterruped = NULL;
	}

	if(pEcorePipeHandlerForSeekComplete)
	{
		ecore_pipe_del(pEcorePipeHandlerForSeekComplete);
		pEcorePipeHandlerForSeekComplete = NULL;
	}

	MpVideoAppCtrlStop(pAppData);
	MpVideoCtrlMgrMediaDestroy(pAppData);
	MpUtilDeletePopupHandle();
	MpVideoViewCtrlDestroy((void *)pAppData);
	MpUtilSetVolumeChangeCbUnset();

	if(MpVideoServiceCtrlGetPlayerType() == MP_STREAMING_PLAYER)
	{
		MpVideoStreamingCtrlDestroy((void *)pAppData);
	}

	MpExternalUgDestroyAll();

	elm_win_indicator_mode_set(pAppData->pMainWindow, ELM_WIN_INDICATOR_SHOW);

	elm_exit();
}

static void MpVideoAppCtrlStartStreaming(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	char *szCookie = MpVideoServiceCtrlGetCookieForStreaming();
	if(szCookie)
	{
		strncpy(pAppData->szCookie, szCookie, STR_LEN_MAX - 1);
	}

	MpVideoCtrlMgrMediaCreate(pAppData->szMediaUri, (void *)MpUtilGetMainWindowXID(),
				NULL, NULL, (void *)pAppData);

	MpVideoCtrlMgrSetPlayerCallback(MpVideoAppCtrlPlayerCompletedCb,
					MpVideoAppCtrlPlayerInterruptedCb,
					MpVideoAppCtrlPlayerErrorCb,
					MpVideoAppCtrlPlayerBufferingCb,
					MpVideoAppCtrlPlayerUpdateSubtitleCb,
					(void*)pAppData);

	MpVideoCtrlMgrMediaRealizeAsync(MpVideoAppCtrlPlayerPrepareCb, (void *)pAppData);

	MpVideoAppCtrlRotateScreen(pAppData, MpUtilGetRotationState());
}

static void MpVideoAppCtrlReLaunching(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	MpVideoAppCtrlStop(pAppData);
	MpVideoCtrlMgrMediaDestroy(pAppData);

	MpVideoViewCtrlHideVolume();

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();
	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		pAppData->nSetPosition = 0;
		MpVideoViewCtrlChangeTitle(pAppData);
	}

	if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoAppCtrlStartStreaming(pAppData);
	}
}

static void MpVideoAppCtrlPipeInit(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return;
	}

	VideoLogInfo("");

	if(pEcorePipeHandlerForSeekComplete)
	{
		ecore_pipe_del(pEcorePipeHandlerForSeekComplete);
		pEcorePipeHandlerForSeekComplete = NULL;
	}
	pEcorePipeHandlerForSeekComplete = ecore_pipe_add((Ecore_Pipe_Cb)MpVideoAppCtrlSeekCompletePipeCb, (void *)pAppData);

	if(pEcorePipeHandleForErrorMsg)
	{
		ecore_pipe_del(pEcorePipeHandleForErrorMsg);
		pEcorePipeHandleForErrorMsg = NULL;
	}

	pEcorePipeHandleForErrorMsg = ecore_pipe_add((Ecore_Pipe_Cb)MpVideoAppCtrlErrormsgCb, (void *)pAppData);

	if(pEcorePipeHandleBOF)
	{
		ecore_pipe_del(pEcorePipeHandleBOF);
		pEcorePipeHandleBOF = NULL;
	}

	pEcorePipeHandleBOF = ecore_pipe_add((Ecore_Pipe_Cb)MpVideoAppCtrlBeginOfPlayCb, (void *)pAppData);

	if(pEcorePipeHandleEOF)
	{
		ecore_pipe_del(pEcorePipeHandleEOF);
		pEcorePipeHandleEOF = NULL;
	}

	pEcorePipeHandleEOF = ecore_pipe_add((Ecore_Pipe_Cb)MpVideoAppCtrlEndOfPlayCb, (void *)pAppData);

	if(pEcorePipeHandleForBuffering)
	{
		ecore_pipe_del(pEcorePipeHandleForBuffering);
		pEcorePipeHandleForBuffering = NULL;
	}

	pEcorePipeHandleForBuffering = ecore_pipe_add((Ecore_Pipe_Cb)MpVideoAppCtrlBufferingCb, (void *)pAppData);

	if(pEcorePipeHandleForResumed)
	{
		ecore_pipe_del(pEcorePipeHandleForResumed);
		pEcorePipeHandleForResumed = NULL;
	}

	pEcorePipeHandleForResumed = ecore_pipe_add((Ecore_Pipe_Cb)MpVideoAppCtrlResumeByRewCb, (void *)pAppData);

	if(pEcorePipeHandleForInterruped)
	{
		ecore_pipe_del(pEcorePipeHandleForInterruped);
		pEcorePipeHandleForInterruped = NULL;
	}

	pEcorePipeHandleForInterruped = ecore_pipe_add((Ecore_Pipe_Cb)MpVideoAppCtrlPauseByInterrupteCb, (void *)pAppData);
}

static void MpVideoAppCtrlSetMute(VideoAppData *pAppData, bool bMute)
{
	VideoLogInfo("");

	MpVideoViewCtrlSetMuteValue(bMute);
}

static void MpVideoAppCtrlShowVolume(VideoAppData *pAppData)
{
	VideoLogInfo("");

	MpVideoViewCtrlShowVolume();
}

static void MpVideoAppCtrlHideVolume(VideoAppData *pAppData)
{
	VideoLogInfo("");

	MpVideoViewCtrlHideVolume();
}


/*
 * External function
 */
bool MpVideoAppCtrlInit(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("No exist appdata.");
		return FALSE;
	}

	VideoLogInfo("");

	VideoLogInfo("Get rotation state from appcore. - %d", MpUtilGetRotationStateFromAppcore());

	MpUtilSetVolumeChangeCbSet((void *)pAppData);

	MpVideoAppCtrlFuncInit(pAppData);
	MpVideoAppCtrlPipeInit(pAppData);
	MpVideoViewCtrlInit(pAppData);

	evas_object_show(pAppData->pMainWindow);
	elm_win_activate(pAppData->pMainWindow);

	if(!MpUtilIsCallOff())
	{
		MpUtilNotifyPopUp(pAppData, _(MP_VPL_BODY_UNABLE_TO_PLAY_VIDEO_DURING_CALL), MpVideoAppCtrlExitCb);
		return FALSE;
	}

	MpVideoAppCtrlInitCheckCallStatus();

	if(!MpUtilCheckBatteryStatus())
	{
		MpUtilNotifyPopUp(pAppData, MP_COM_BODY_LOW_BATTERY, MpVideoAppCtrlExitCb);
		return FALSE;
	}

	MpVideoAppCtrlStart(pAppData);

	return TRUE;
}

bool MpVideoAppCtrlResetForRelaunching(VideoAppData *pAppData)
{
	if (!pAppData) {
		VideoLogInfo("No exist appdata.");
		return FALSE;
	}

	VideoLogInfo("");

	MpVideoAppCtrlStop(pAppData);

	MpVideoCtrlMgrMediaDestroy(pAppData);
	MpUtilDeletePopupHandle();
	MpVideoViewCtrlDestroy((void *)pAppData);

	MpUtilMediaSvcFinishSession();
	MpUtilBatteryStatusNotiIgnore();

	nBufferingRate = 0;
	nPlayerErrorType = 0;
	nPlayerInterruptType = 0;

	pAppData->nDuration = 0;
	pAppData->nSetPosition = 0;
	pAppData->nCurPosition = 0;
	pAppData->nBufferingPosition = 0;
	pAppData->nVideoWidthResolution = 0;
	pAppData->nVideoHeightResolution = 0;

	memset(pAppData->szVideoTitle, 0, sizeof(char) * STR_LEN_MAX);
	memset(pAppData->szMediaUri, 0, sizeof(char) * STR_LEN_MAX);
	memset(pAppData->szPreMediaUri, 0, sizeof(char) * STR_LEN_MAX);
	memset(pAppData->szNextMediaUri, 0, sizeof(char) * STR_LEN_MAX);

	pAppData->nPrePlayerState = MP_PLAYER_STATE_CLEAR;
	pAppData->nCurPlayerState = MP_PLAYER_STATE_CLEAR;
	pAppData->nPlayingSpeed = 0;

	memset(pAppData->szProxyAddress, 0, sizeof(char) * STR_LEN_MAX);
	memset(pAppData->szUserAgent, 0, sizeof(char) * STR_LEN_MAX);
	memset(pAppData->szCookie, 0, sizeof(char) * STR_LEN_MAX);

	MpUtilResetForRelaunching();

	MpVideoAppCtrlInitCheckCallStatus();

	return TRUE;
}

bool MpVideoAppCtrlIsTopView(void* pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist pUserData.");
		return FALSE;
	}

	VideoLogInfo("");

	return MpVideoViewCtrlIsTopView(pUserData);
}

bool MpVideoAppCtrlIsExistPlayerHandle(void)
{
	VideoLogInfo("");

	return MpVideoCtrlMgrIsExistPlayerHandle();
}

void MpVideoAppCtrlSetForCheckingForegroundwindow(bool bForwgroundState)
{
	VideoLogInfo("");

	bWindowForegroundState = bForwgroundState;
}

void MpVideoAppCtrlSetRotation(int nRotateMode, void *pUserData)
{
	VideoLogInfo("");

	MpVideoViewCtrlSetRotation(nRotateMode, TRUE, pUserData);
}

bool MpVideoAppCtrlIsCheckCallStatus(void)
{
	VideoLogInfo("");

	bool bCheckCallStatus = FALSE;

	if(g_nCurrentInterrupt == MP_PLAYER_INTERRUPTED_BY_CALL)
	{
		bCheckCallStatus = TRUE;
	}

	return bCheckCallStatus;

}
void MpVideoAppCtrlInitCheckCallStatus(void)
{
	VideoLogInfo("");

	g_nCurrentInterrupt = MP_PLAYER_INTERRUPTED_COMPLETED;
}

