/*
 * Copyright (c) [2012] Samsung Electronics Co., Ltd.
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <aul.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <syspopup_caller.h>

#include "mp-util.h"
#include "mp-video-log.h"
#include "video-player.h"
#include "mp-video-display-view.h"
#include "mp-video-progressbar.h"
#include "mp-video-type-define.h"
#include "mp-video-value-define.h"
#include "mp-util-media-service.h"
#include "mp-video-string-define.h"
#include "mp-video-sound-path-ctrl.h"

#include "mp-external-ug.h"
#include "mp-video-service-ctrl.h"

#define CTRL_PANEL_SHOW_TIME					5.0
#define MAX_SPEED_STEP							4
#define REWIND_SPACE_TIME						3000

static Evas_Object *pNaviFrameHandle = NULL;
static Evas_Object *pDisplayViewLayout = NULL;
static Evas_Object *pDisplayViewPanelEdj = NULL;
static Evas_Object *pDisplayViewCtrlEdj = NULL;
static Evas_Object *pDisplayViewOptionEdj = NULL;
static Evas_Object *pDisplayViewProgressbarEdj = NULL;
static Evas_Object *pDisplaySoundpathButton = NULL;

static Evas_Object *pDisplayViewSpeedboxEdj = NULL;

static Ecore_Timer *pCtrlPanelHideTimer = NULL;
static Ecore_Timer *pSpeedPressTimer = NULL;
static Elm_Object_Item *pNaviFrameItemForDisplayerView = NULL;
static ChangeDisplayViewCbFunc pChangeViewUserCbFunc = NULL;

static bool bSwitchDisplayViewUI = TRUE;
static bool bIsSpeedLongPress = FALSE;
static bool bIsDisplayViewOnlyAudio = FALSE;
static int nSpeedMode = 0;	/*  (-1:REW, 1: FF) */
static int nControlMouseDownPos = 0;
static int nControlMouseMovePos = 0;
static bool bIsPauseByUser = FALSE;
static bool bIsBeginOfStream = FALSE;
static VideoDisplayViewState nDisplayViewState = MP_DISPLAY_VIEW_STATE_UNACTIVATE;


static void MpVideoDisplayViewWarningPopupCb(void *pUserData, Evas_Object *pObject, void *pEventInfo)
{
	if (!pUserData) {
		VideoLogInfo("No exist pUserData");
		return;
	}

	VideoLogInfo("");

	MpUtilDeletePopupHandle();
}


static void MpVideoDisplayViewSoundPathCb(void *pUserData, Evas_Object *pObject, void *pEventInfo)
{
	if (!pUserData) {
		VideoLogInfo("[ERR] No exist pUserData");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (MpVideoSoundpathIsConnectOtherSoundpath()) {
		VideoLogInfo("Connect BT or earjack.");
		MpVideoSoundpathShow(pUserData);
	} else {
		MpUtilNotifyPopUp(pAppData, "Soundpath is connected with speaker only.", MpVideoDisplayViewWarningPopupCb);
	}
}


static void MpVideoDisplayViewPressOptionShareCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(!bIsBeginOfStream)
	{
		VideoLogInfo("Skip for BOF");
		return;
	}

	if(pCtrlPanelHideTimer)
	{
		ecore_timer_del(pCtrlPanelHideTimer);
		pCtrlPanelHideTimer = NULL;
	}

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause(pAppData);

	MpExternalUgShow((void *)pAppData, pNaviFrameHandle);
}

static void MpVideoDisplayViewPressPauseButtonCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	if(!bIsBeginOfStream)
	{
		VideoLogInfo("Skip for BOF");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(!strcmp(szEmission, SIGNAL_CONTROL_PLAY_PAUSE_PRESS))
	{
		VideoLogInfo("Press Pause button.");

		if(pAppData->bSeekComplete == FALSE)
		{
			bIsPauseByUser = TRUE;
			pAppData->nSeekPlayerState = MP_PLAYER_STATE_PAUSE;
			return;
		}

		if (pAppData->nCurPlayerState == MP_PLAYER_STATE_RESUME || pAppData->nCurPlayerState == MP_PLAYER_STATE_PLAY)
		{
			bIsPauseByUser = TRUE;
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause(pAppData);
		}
	}
}

static void MpVideoDisplayViewPressResumeButtonCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	if(!bIsBeginOfStream)
	{
		VideoLogInfo("Skip for BOF");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (!strcmp(szEmission, SIGNAL_CONTROL_PLAY_RESUME_PRESS)) {
		VideoLogInfo("Press Resume/Play button.");

		if(pAppData->bSeekComplete == FALSE)
		{
			bIsPauseByUser = FALSE;
			pAppData->nSeekPlayerState = MP_PLAYER_STATE_RESUME;
			return;
		}

		if (pAppData->nCurPlayerState != MP_PLAYER_STATE_RESUME && pAppData->nCurPlayerState != MP_PLAYER_STATE_PLAY)
		{
			bIsPauseByUser = FALSE;
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlResume(pAppData);
		}
	}
}

static Eina_Bool MpVideoDisplayViewSpeedLongPressTimerCb(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return EINA_FALSE;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (pSpeedPressTimer) {
		ecore_timer_del(pSpeedPressTimer);
		pSpeedPressTimer = NULL;
	}

	bIsSpeedLongPress = TRUE;
	pAppData->nPlayingSpeed = MP_PLAYING_SPEED_2X;

	char caption[CAPTION_TEXT_MAX_LEN] = { 0, };

	if(nSpeedMode == 1)
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedFF(pAppData);
		snprintf(caption, CAPTION_TEXT_MAX_LEN, "x%d", pAppData->nPlayingSpeed);

		edje_object_part_swallow(pDisplayViewCtrlEdj, PART_CONTROL_SPEED_BOX_FF, pDisplayViewSpeedboxEdj);
		evas_object_show(pDisplayViewSpeedboxEdj);
	}
	else if(nSpeedMode == -1)
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedRew(pAppData);
		snprintf(caption, CAPTION_TEXT_MAX_LEN, "x%d", pAppData->nPlayingSpeed);
		edje_object_part_swallow(pDisplayViewCtrlEdj, PART_CONTROL_SPEED_BOX_REW, pDisplayViewSpeedboxEdj);
		evas_object_show(pDisplayViewSpeedboxEdj);
	}

	edje_object_signal_emit(pDisplayViewSpeedboxEdj, SIGNAL_SPEEDBOX_SHOW, "*");
	edje_object_signal_emit(pDisplayViewSpeedboxEdj, SIGNAL_SPEEDBOX_SET_MODE_1, "*");

	edje_object_part_text_set(_EDJ(pDisplayViewLayout), MAIN_SPEED_TEXT, caption);
	edje_object_signal_emit(_EDJ(pDisplayViewLayout), SIGNAL_MAIN_SPEEDBOX_SHOW, "*");

	return EINA_FALSE;
}


static void MpVideoDisplayViewFFDownButtonCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (!strcmp(szEmission, SIGNAL_CONTROL_NORMAL_FF_BUTTON_DOWN)) {
		VideoLogInfo("Press FF down button.");
		if (pCtrlPanelHideTimer) {
			ecore_timer_del(pCtrlPanelHideTimer);
			pCtrlPanelHideTimer = NULL;
		}

		if (pSpeedPressTimer) {
			ecore_timer_del(pSpeedPressTimer);
			pSpeedPressTimer = NULL;
		}

		nSpeedMode = 1;
		pSpeedPressTimer = ecore_timer_add(LONG_PRESS_INTERVAL, MpVideoDisplayViewSpeedLongPressTimerCb, (void *)pAppData);
	}
}

static void MpVideoDisplayViewFFUpButtonCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(!bIsBeginOfStream)
	{
		VideoLogInfo("Skip for BOF");
		return;
	}

	if (!strcmp(szEmission, SIGNAL_CONTROL_NORMAL_FF_BUTTON_UP)) {
		VideoLogInfo("Press FF up button.");
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowLayout(pAppData);

		if (pSpeedPressTimer) {
			ecore_timer_del(pSpeedPressTimer);
			pSpeedPressTimer = NULL;
			bIsPauseByUser = FALSE;
		}
		else
		{
			edje_object_part_text_set(_EDJ(pDisplayViewLayout), MAIN_SPEED_TEXT, "");
			edje_object_signal_emit(_EDJ(pDisplayViewLayout), SIGNAL_MAIN_SPEEDBOX_HIDE, "*");

			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedReset(pAppData);
		}
		edje_object_signal_emit(pDisplayViewSpeedboxEdj, SIGNAL_SPEEDBOX_HIDE, "*");
		edje_object_part_unswallow(pDisplayViewCtrlEdj, pDisplayViewSpeedboxEdj);
		evas_object_hide(pDisplayViewSpeedboxEdj);


		bIsSpeedLongPress = FALSE;
	}
}

static void MpVideoDisplayViewFFClickedButtonCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(!bIsBeginOfStream)
	{
		VideoLogInfo("Skip for BOF");
		return;
	}

	if(!strcmp(szEmission, SIGNAL_CONTROL_NORMAL_FF_BUTTON_CLICKED))
	{
		VideoLogInfo("Press FF clicked button.");

		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlGetPosition(pAppData);

		bIsPauseByUser = FALSE;
	}
}

static void MpVideoDisplayViewRewDownButtonCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	if(!bIsBeginOfStream)
	{
		VideoLogInfo("Skip for BOF");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (!strcmp(szEmission, SIGNAL_CONTROL_NORMAL_REW_BUTTON_DOWN)) {
		VideoLogInfo("Press Rewind donw button.");

		if (pCtrlPanelHideTimer) {
			ecore_timer_del(pCtrlPanelHideTimer);
			pCtrlPanelHideTimer = NULL;
		}

		if (pSpeedPressTimer) {
			ecore_timer_del(pSpeedPressTimer);
			pSpeedPressTimer = NULL;
		}

		nSpeedMode = -1;
		pSpeedPressTimer = ecore_timer_add(LONG_PRESS_INTERVAL, MpVideoDisplayViewSpeedLongPressTimerCb, (void *)pAppData);
	}
}

static void MpVideoDisplayViewRewUpButtonCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	if(!bIsBeginOfStream)
	{
		VideoLogInfo("Skip for BOF");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (!strcmp(szEmission, SIGNAL_CONTROL_NORMAL_REW_BUTTON_UP)) {
		VideoLogInfo("Press Rewind up button.");
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowLayout(pAppData);

		if (pSpeedPressTimer) {
			ecore_timer_del(pSpeedPressTimer);
			pSpeedPressTimer = NULL;
			bIsPauseByUser = FALSE;
		}
		else
		{
			edje_object_part_text_set(_EDJ(pDisplayViewLayout), MAIN_SPEED_TEXT, "");
			edje_object_signal_emit(_EDJ(pDisplayViewLayout), SIGNAL_MAIN_SPEEDBOX_HIDE, "*");

			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedReset(pAppData);
		}

		edje_object_signal_emit(pDisplayViewSpeedboxEdj, SIGNAL_SPEEDBOX_HIDE, "*");
		evas_object_hide(pDisplayViewSpeedboxEdj);
		edje_object_part_unswallow(pDisplayViewCtrlEdj, pDisplayViewSpeedboxEdj);


		bIsSpeedLongPress = FALSE;
	}
}

static void MpVideoDisplayViewRewClickedButtonCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	if(!bIsBeginOfStream)
	{
		VideoLogInfo("Skip for BOF");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (!strcmp(szEmission, SIGNAL_CONTROL_NORMAL_REW_BUTTON_CLICKED)) {
		VideoLogInfo("Press Rewind clicked button.");

		if(pAppData->nCurPosition > REWIND_SPACE_TIME)
		{
			pAppData->nSetPosition = 0;
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSetPosition(pAppData);
		}
		else
		{
			bIsPauseByUser = FALSE;
		}
 	}
}

static void MpVideoDisplayViewBackButtonDownCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	edje_object_signal_callback_del(pDisplayViewCtrlEdj, SIGNAL_CONTROL_BACK_BUTTON_DOWN, SIGNAL_CONTROL_PART_BACK_BUTTON, (void *)MpVideoDisplayViewBackButtonDownCb);

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(pAppData->nCurPosition >= 0)
	{
		MpUtilMediaSvcSetVideoLastPlayedTimeByUri(pAppData->szMediaUri, pAppData->nCurPosition);
	}

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlStopByAppPause(pAppData);

	MpVideoDisplayViewPop(pNaviFrameHandle, pUserData);

	edje_object_signal_callback_add(pDisplayViewCtrlEdj, SIGNAL_CONTROL_BACK_BUTTON_DOWN, SIGNAL_CONTROL_PART_BACK_BUTTON, (void *)MpVideoDisplayViewBackButtonDownCb, (void *)pAppData);
}


static void MpVideoDisplayViewVolumeButtonDownCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");
}

static void MpVideoDisplayViewVolumeButtonUpCb(void *pUserData,
					       Evas_Object *pObject,
					       const char *szEmission,
					       const char *szSource)
{
	VideoLogInfo("");
	if (!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowVolume(pAppData);
}

static Eina_Bool MpVideoDisplayViewCtrlPanelHideCb(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR] No Exist pUserData.");
		return EINA_FALSE;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (pCtrlPanelHideTimer) {
		ecore_timer_del(pCtrlPanelHideTimer);
		pCtrlPanelHideTimer = NULL;
	}

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideLayout(pAppData);

	return EINA_FALSE;
}


static void MpVideoDisplayViewSpeedChange(void *pUserData)
{
	if (!pUserData) {
		VideoLogInfo("[ERR] No Exist pUserData.");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	int booster_step = 0;

	if (nSpeedMode == 1) {
		booster_step = nControlMouseDownPos - nControlMouseMovePos;
		booster_step = booster_step / (BOOSTER_STEP_DISTANCE * elm_config_scale_get());
	} else if (nSpeedMode == -1) {
		booster_step = nControlMouseDownPos - nControlMouseMovePos;
		booster_step = booster_step / (BOOSTER_STEP_DISTANCE * elm_config_scale_get());
	} else {
		VideoLogInfo("[ERR] Invalid SpeedMode.");
		return;
	}


	if(booster_step == 0)
	{
		pAppData->nPlayingSpeed = MP_PLAYING_SPEED_2X;
		edje_object_signal_emit(pDisplayViewSpeedboxEdj, SIGNAL_SPEEDBOX_SET_MODE_1, "*");
	}
	else if(booster_step == 1)
	{
		pAppData->nPlayingSpeed = MP_PLAYING_SPEED_3X;
		edje_object_signal_emit(pDisplayViewSpeedboxEdj, SIGNAL_SPEEDBOX_SET_MODE_2, "*");
	}
	else if(booster_step > 2)
	{
		pAppData->nPlayingSpeed = MP_PLAYING_SPEED_4X;
		edje_object_signal_emit(pDisplayViewSpeedboxEdj, SIGNAL_SPEEDBOX_SET_MODE_3, "*");
	}

	VideoLogInfo("STEP:(%d, %d)", booster_step, pAppData->nPlayingSpeed);

	char caption[CAPTION_TEXT_MAX_LEN] = { 0, };

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlGetPosition(pAppData);

	if(nSpeedMode == 1)
	{
		snprintf(caption, CAPTION_TEXT_MAX_LEN, "x%d", pAppData->nPlayingSpeed);
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedFF(pAppData);
	}
	else
	{
		snprintf(caption, CAPTION_TEXT_MAX_LEN, "x%d", pAppData->nPlayingSpeed);
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedRew(pAppData);
	}

	edje_object_part_text_set(_EDJ(pDisplayViewLayout), MAIN_SPEED_TEXT, caption);

	return;
}

static void MpVideoDisplayViewControlMouseMoveCb(void *pUserData, Evas * e, Evas_Object *obj, void *event)
{
	if (!event) {
		VideoLogInfo("[ERR]");
		return;
	}

	if (!pUserData) {
		VideoLogInfo("[ERR] No Exist pUserData.");
		return;
	}

	Evas_Event_Mouse_Move *pBtnEvent = (Evas_Event_Mouse_Move *) event;

	if (bIsSpeedLongPress) {
		nControlMouseMovePos = pBtnEvent->cur.output.y;
		MpVideoDisplayViewSpeedChange(pUserData);
	}
}

static void MpVideoDisplayViewControlMouseDownCb(void *pUserData, Evas * e, Evas_Object *obj, void *event)
{
	VideoLogInfo("");

	if (!event) {
		VideoLogInfo("[ERR]");
		return;
	}

	Evas_Event_Mouse_Down *pBtnEvent = (Evas_Event_Mouse_Down *) event;

	nControlMouseDownPos = pBtnEvent->output.y;
}

static void MpVideoDisplayViewChangedEarjackCb(void *pNode, void *pUserData)
{
	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	MpVideoSoundpathHide();
	MpUtilDeletePopupHandle();

	if (MpUtilIsConnectEarjack() || MpVideoSoundpathIsConnectOtherSoundpath()) {
		VideoLogInfo("Connect earjack.");
		if (pDisplaySoundpathButton) {
			elm_object_item_part_content_set(pNaviFrameItemForDisplayerView, "title_right_btn", pDisplaySoundpathButton);
		} else {
			pDisplaySoundpathButton = MpUtilCreateButtonIcon(pNaviFrameHandle, VIDEO_HEAD_ICON_SOUNDPATH, FALSE, FALSE, 1, 1, pUserData, "video/naviframe/title/default", (void*)MpVideoDisplayViewSoundPathCb);
			elm_object_item_part_content_set(pNaviFrameItemForDisplayerView, "title_right_btn", pDisplaySoundpathButton);
		}
	} else {
		VideoLogInfo("Disconnect earjack.");

		if (pDisplaySoundpathButton) {
			evas_object_del(pDisplaySoundpathButton);
			pDisplaySoundpathButton = NULL;
		}

		elm_object_item_part_content_unset(pNaviFrameItemForDisplayerView, "title_right_btn");
	}
}

static void MpVideoDisplayViewChangedCameraStatusCb(void *pNode, void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(!MpUtilIsCameraOff())
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}
}
static void MpVideoDisplayViewHideLayoutFinish(void *pUserData, Evas_Object *pObj, const char *emission, const char *source)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogError("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideFinish(pAppData);

	evas_object_hide(pDisplayViewLayout);
}

static void MpVideoDisplayViewShowLayoutFinish(void *pUserData, Evas_Object *pObj, const char *emission, const char *source)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogError("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowFinish(pAppData);
}

void MpVideoDisplayViewInitLayout(void *pUserData, int nPlayerType)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pDisplayViewLayout = elm_layout_add(pNaviFrameHandle);
	if (pDisplayViewLayout) {
		Evas *pMainWindowEvas = evas_object_evas_get(pAppData->pMainWindow);

		elm_layout_file_set(pDisplayViewLayout, VIDEO_PLAYER_MAIN_EDJ, MAIN_EDJ_GROUP);

		pDisplayViewOptionEdj = MpUtilLoadEdjFile(pMainWindowEvas, VIDEO_PLAYER_OPTION_BUTTON_EDJ, OPTION_EDJ_GROUP);
		elm_object_part_content_set(pDisplayViewLayout, SWALLOW_MAIN_OPTION_BUTTON, pDisplayViewOptionEdj);

		edje_object_signal_emit(pDisplayViewOptionEdj, SIGNAL_OPTION_MODE_LOCAL, "*");

		pDisplayViewCtrlEdj = MpUtilLoadEdjFile(pMainWindowEvas, VIDEO_PLAYER_CONTROL_EDJ, CONTROL_EDJ_GROUP);

		evas_object_event_callback_add(pDisplayViewCtrlEdj, EVAS_CALLBACK_MOUSE_DOWN, (void *)MpVideoDisplayViewControlMouseDownCb, pAppData);
		evas_object_event_callback_add(pDisplayViewCtrlEdj, EVAS_CALLBACK_MOUSE_MOVE, (void *)MpVideoDisplayViewControlMouseMoveCb, pAppData);

		if(nPlayerType == MP_VIDEO_PLAYER_GALLERY)
		{
			int width = 0;
			int height = 0;

			int item_width = 120;
			int item_height = 90;

			ecore_x_window_size_get(ecore_x_window_root_first_get(),&width, &height);
			if(width > 0) {
				item_width = 170 * width/DEFAULT_SCALE_WIDTH;
			}

			if(height > 0) {
				item_height = 145 * height/DEFAULT_SCALE_HEIGHT;
			}
		}

		pDisplayViewPanelEdj = MpUtilLoadEdjFile(pMainWindowEvas, VIDEO_PLAYER_PANEL_EDJ, PANEL_EDJ_GROUP);

		edje_object_part_swallow(pDisplayViewPanelEdj, SWALLOW_PANEL_CONTROL, pDisplayViewCtrlEdj);

		pDisplayViewProgressbarEdj = (Evas_Object *)MpVideoProgressbarInit(pAppData);
		if (pDisplayViewProgressbarEdj) {
			edje_object_part_swallow(pDisplayViewPanelEdj, SWALLOW_PANEL_PROGRESS_BAR, pDisplayViewProgressbarEdj);
		} else {
			VideoLogInfo("Progressbar edj handle is null.");
		}

		elm_object_part_content_set(pDisplayViewLayout, SWALLOW_MAIN_CONTROL, pDisplayViewPanelEdj);
		edje_object_signal_emit(pDisplayViewCtrlEdj, SIGNAL_CONTROL_MODE_NORMAL, "*");

		edje_object_signal_callback_add(pDisplayViewOptionEdj,
					SIGNAL_OPTION_SHARE_DOWN,
					PART_OPTION_SHARE,
					(void *)MpVideoDisplayViewPressOptionShareCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_PLAY_PAUSE_PRESS,
						SIGNAL_CONTROL_PART_PLAY_BUTTON,
						(void *)MpVideoDisplayViewPressPauseButtonCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_PLAY_RESUME_PRESS,
						SIGNAL_CONTROL_PART_PLAY_BUTTON,
						(void *)MpVideoDisplayViewPressResumeButtonCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_NORMAL_FF_BUTTON_DOWN,
						SIGNAL_CONTROL_PART_FF_BUTTON,
						(void *)MpVideoDisplayViewFFDownButtonCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_NORMAL_FF_BUTTON_UP,
						SIGNAL_CONTROL_PART_FF_BUTTON,
						(void *)MpVideoDisplayViewFFUpButtonCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_NORMAL_FF_BUTTON_CLICKED,
						SIGNAL_CONTROL_PART_FF_BUTTON,
						(void *)MpVideoDisplayViewFFClickedButtonCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_NORMAL_REW_BUTTON_DOWN,
						SIGNAL_CONTROL_PART_REW_BUTTON,
						(void *)MpVideoDisplayViewRewDownButtonCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_NORMAL_REW_BUTTON_UP,
						SIGNAL_CONTROL_PART_REW_BUTTON,
						(void *)MpVideoDisplayViewRewUpButtonCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_NORMAL_REW_BUTTON_CLICKED,
						SIGNAL_CONTROL_PART_REW_BUTTON,
						(void *)MpVideoDisplayViewRewClickedButtonCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_BACK_BUTTON_DOWN,
						SIGNAL_CONTROL_PART_BACK_BUTTON,
						(void *)MpVideoDisplayViewBackButtonDownCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_VOL_BUTTON_DOWN,
						SIGNAL_CONTROL_PART_VOL_BUTTON,
						(void *)MpVideoDisplayViewVolumeButtonDownCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_VOL_BUTTON_UP,
						SIGNAL_CONTROL_PART_VOL_BUTTON,
						(void *)MpVideoDisplayViewVolumeButtonUpCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_MUTE_BUTTON_DOWN,
						SIGNAL_CONTROL_PART_MUTE_BUTTON,
						(void *)MpVideoDisplayViewVolumeButtonDownCb, (void *)pAppData);

		edje_object_signal_callback_add(pDisplayViewCtrlEdj,
						SIGNAL_CONTROL_MUTE_BUTTON_UP,
						SIGNAL_CONTROL_PART_MUTE_BUTTON,
						(void *)MpVideoDisplayViewVolumeButtonUpCb, (void *)pAppData);

		pDisplayViewSpeedboxEdj = MpUtilLoadEdjFile(pMainWindowEvas, VIDEO_PLAYER_SPEEDBOX_VIEW_EDJ, SPEEDBOX_EDJ_GROUP);
		evas_object_hide(pDisplayViewSpeedboxEdj);

		MpUtilRegisteEarjackCbFunc((void *)MpVideoDisplayViewChangedEarjackCb, pUserData);
		MpUtilRegisteCameraStatusCbFunc((void *)MpVideoDisplayViewChangedCameraStatusCb, pUserData);
	}
}

void MpVideoDisplayViewInit(void *pUserData, int nPlayerType)
{
	VideoLogInfo("");

	if (!pNaviFrameHandle|| !pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}
	bIsPauseByUser = FALSE;

	MpVideoDisplayViewInitLayout(pUserData, nPlayerType);
}


/*
 * External function
 */
void MpVideoDisplayViewPush(void *pNaviFrame, int nPlayerType, void *pUserData, ChangeDisplayViewCbFunc pChangeViewCb)
{
	if (!pNaviFrame || !pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pNaviFrameHandle = pNaviFrame;
	pChangeViewUserCbFunc = pChangeViewCb;

	MpVideoDisplayViewInit(pUserData, nPlayerType);
	pNaviFrameItemForDisplayerView = elm_naviframe_item_push(pNaviFrameHandle, NULL, NULL, NULL, pDisplayViewLayout, "video/1line");

	elm_object_signal_callback_add(pDisplayViewLayout, SIGNAL_CONTORL_SHOW_FINISH, "*", MpVideoDisplayViewShowLayoutFinish, pUserData);
	elm_object_signal_callback_add(pDisplayViewLayout, SIGNAL_CONTORL_HIDE_FINISH, "*", MpVideoDisplayViewHideLayoutFinish, pUserData);

	if(MpVideoSoundpathIsConnectOtherSoundpath())
	{
		pDisplaySoundpathButton = MpUtilCreateButtonIcon(pNaviFrameHandle, VIDEO_HEAD_ICON_SOUNDPATH, FALSE, FALSE, 1, 1, pUserData, "video/naviframe/title/default", (void*)MpVideoDisplayViewSoundPathCb);
		elm_object_item_part_content_set(pNaviFrameItemForDisplayerView, "title_right_btn", pDisplaySoundpathButton);
	}

	if(MpUtilCheckMuteStatus())
	{
		MpVideoDisplayViewSetMuteStatus(TRUE);
	}

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowLayout(pAppData);
}

void MpVideoDisplayViewPop(void *pNaviFrame, void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideVolume(pAppData);
	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
}

void MpVideoDisplayViewDestroy(void *pUserData)
{
	VideoLogInfo("");

	MpVideoDisplayViewHideControlPanel(pUserData);
	MpVideoProgressbarDestroy();

	pNaviFrameHandle = NULL;
	pDisplayViewProgressbarEdj = NULL;

	if (pDisplayViewPanelEdj) {
		evas_object_del(pDisplayViewPanelEdj);
		pDisplayViewPanelEdj = NULL;
	}

	if (pDisplayViewCtrlEdj) {
		evas_object_del(pDisplayViewCtrlEdj);
		pDisplayViewCtrlEdj = NULL;
	}

	if (pDisplayViewOptionEdj) {
		evas_object_del(pDisplayViewOptionEdj);
		pDisplayViewOptionEdj = NULL;
	}

	if (pSpeedPressTimer) {
		ecore_timer_del(pSpeedPressTimer);
		pSpeedPressTimer = NULL;
	}

	if (pCtrlPanelHideTimer) {
		ecore_timer_del(pCtrlPanelHideTimer);
		pCtrlPanelHideTimer = NULL;
	}

	if (pDisplayViewLayout) {
		evas_object_del(pDisplayViewLayout);
		pDisplayViewLayout = NULL;
	}

	if (pDisplaySoundpathButton) {
		evas_object_del(pDisplaySoundpathButton);
		pDisplaySoundpathButton = NULL;
	}

	if (pDisplayViewSpeedboxEdj) {
		evas_object_del(pDisplayViewSpeedboxEdj);
		pDisplayViewSpeedboxEdj = NULL;
	}

	bSwitchDisplayViewUI = TRUE;
	nDisplayViewState = MP_DISPLAY_VIEW_STATE_UNACTIVATE;
	bIsDisplayViewOnlyAudio = FALSE;
	bIsPauseByUser = FALSE;
}

void MpVideoDisplayViewInitAfterBeginOfStream(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	bIsBeginOfStream = TRUE;

	pAppData->bSeekComplete = TRUE;

	MpVideoProgressbarBeginOfStream((void *)pAppData);

	if (pAppData->nDuration > 1) {
		char szDurationTime[PLAY_TIME_LENGTH] = { 0 };

		snprintf(szDurationTime, PLAY_TIME_LENGTH, "%" PLAY_TIME_FORMAT, PLAY_TIME_ARGS(pAppData->nDuration / 1000));

		MpVideoProgressbarUpdateDurationTime(szDurationTime);
	}

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowLayout(pAppData);

	if(MpVideoDisplayViewIsPauseByUser() == FALSE)
	{
		edje_object_signal_emit(pDisplayViewCtrlEdj, SIGNAL_CONTROL_PLAY_RESUME, "*");
	}
}

void MpVideoDisplayViewUpdatePauseKey(void)
{
	VideoLogInfo("");

	edje_object_signal_emit(pDisplayViewCtrlEdj, SIGNAL_CONTROL_PLAY_PAUSE, "*");
}

void MpVideoDisplayViewUpdateResumeKey(void)
{
	VideoLogInfo("");

	edje_object_signal_emit(pDisplayViewCtrlEdj, SIGNAL_CONTROL_PLAY_RESUME, "*");
}




void MpVideoDisplayViewShowControlPanel(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	if (!pDisplayViewPanelEdj) {
		VideoLogInfo("No exist play control panel edc.");
		return;
	}

	if (nDisplayViewState == MP_DISPLAY_VIEW_STATE_UNACTIVATE) {
		VideoLogInfo("MP_DISPLAY_VIEW_STATE_UNACTIVATE");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(bIsBeginOfStream)
	{
		MpVideoProgressbarCreateTimer((void *)pAppData);
	}

	bSwitchDisplayViewUI = TRUE;
	evas_object_show(pDisplayViewLayout);

	elm_object_item_signal_emit(pNaviFrameItemForDisplayerView, SIGNAL_NAVIFRAME_TITLE_SHOW, "elm");
	int nRotate = MpUtilGetRotationState();
	if( nRotate == VIDEO_ROTATE_PORTRAIT_REVERSE || nRotate == VIDEO_ROTATE_PORTRAIT_NORMAL)
	{
		elm_object_signal_emit(pDisplayViewLayout, SIGNAL_MAIN_CONTROL_PORTRAIT_SHOW, "*");
	}
	else
	{
		elm_object_signal_emit(pDisplayViewLayout, SIGNAL_MAIN_CONTROL_LANDSCAPE_SHOW, "*");
	}

	if (pCtrlPanelHideTimer) {
		ecore_timer_del(pCtrlPanelHideTimer);
		pCtrlPanelHideTimer = NULL;
	}

	VideoLogInfo("");

	if (pAppData->nCurPlayerState == MP_PLAYER_STATE_PAUSE || bIsDisplayViewOnlyAudio == TRUE) {
		VideoLogInfo("");
	} else {
		VideoLogInfo("");
		pCtrlPanelHideTimer = ecore_timer_add(CTRL_PANEL_SHOW_TIME, MpVideoDisplayViewCtrlPanelHideCb, pUserData);
	}

	MpVideoDisplayViewActivate();
}

void MpVideoDisplayViewHideControlPanel(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	if (nDisplayViewState == MP_DISPLAY_VIEW_STATE_UNACTIVATE) {
		VideoLogInfo("MP_DISPLAY_VIEW_STATE_UNACTIVATE");
		return;
	}

	if (!pDisplayViewPanelEdj) {
		VideoLogInfo("No exist play control panel edc.");
		return;
	}

	if(bIsDisplayViewOnlyAudio)
	{
		VideoLogInfo("Not hide control panel and UI because of audio only.");
		return;
	}

	bSwitchDisplayViewUI = FALSE;

	MpVideoProgressbarDeleteTimer();
	MpExternalUgHide();

	if (pCtrlPanelHideTimer) {
		ecore_timer_del(pCtrlPanelHideTimer);
		pCtrlPanelHideTimer = NULL;
	}

	elm_object_item_signal_emit(pNaviFrameItemForDisplayerView, SIGNAL_NAVIFRAME_TITLE_HIDE, "elm");
	elm_object_signal_emit(pDisplayViewLayout, SIGNAL_MAIN_CONTROL_NORMAL_HIDE, "*");
}

void MpVideoDisplayViewActivate(void)
{
	VideoLogInfo("");

	nDisplayViewState = MP_DISPLAY_VIEW_STATE_ACTIVATE;
}

void MpVideoDisplayViewUnactivate(void)
{
	VideoLogInfo("");

	nDisplayViewState = MP_DISPLAY_VIEW_STATE_UNACTIVATE;
}

void MpVideoDisplayViewChangeNaviFrameTitle(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	Elm_Object_Item *pNaviFrameItemForDisplayerView = elm_naviframe_top_item_get(pNaviFrameHandle);

	bIsBeginOfStream = FALSE;
	MpVideoProgressbarDeleteTimer();

	if (pSpeedPressTimer) {
		ecore_timer_del(pSpeedPressTimer);
		pSpeedPressTimer = NULL;
	}

	edje_object_part_text_set(_EDJ(pDisplayViewLayout), MAIN_SPEED_TEXT, "");
	edje_object_signal_emit(_EDJ(pDisplayViewLayout), SIGNAL_MAIN_SPEEDBOX_HIDE, "*");

	if (pNaviFrameItemForDisplayerView != NULL) {
		VideoLogInfo("%s", pAppData->szVideoTitle);
		elm_object_item_text_set(pNaviFrameItemForDisplayerView, pAppData->szVideoTitle);
	}
}

bool MpVideoDisplayViewIsTopView(void)
{
	VideoLogInfo("");

	Evas_Object *pTmpContents = NULL;
	Elm_Object_Item *pTmpItem = NULL;

	if (!pNaviFrameHandle) {
		VideoLogInfo("[ERR] No exist navigationbar handle.");
		return FALSE;
	}

	pTmpItem = elm_naviframe_top_item_get(pNaviFrameHandle);
	pTmpContents = elm_object_item_content_get(pTmpItem);
	if (pTmpContents) {
		if (pTmpContents == pDisplayViewLayout) {
			VideoLogInfo("Video display view is staing in top view.");
			return TRUE;
		} else {
			VideoLogInfo("Video display view is not staied in top view.");
			return FALSE;
		}
	} else {
		VideoLogInfo("[ERR] No exist navigationbar top item.");
		return FALSE;
	}

	return FALSE;
}

void MpVideoDisplayViewShowNocontentImage(void *pUserData)
{
	VideoLogInfo("");

	if (pDisplayViewLayout) {
		edje_object_signal_emit(_EDJ(pDisplayViewLayout), SIGNAL_MAIN_NOCONTENTS_SHOW, "*");
	}

	bIsDisplayViewOnlyAudio = TRUE;

	MpVideoDisplayViewShowControlPanel(pUserData);
}

void MpVideoDisplayViewHideNocontentImage(void)
{
	VideoLogInfo("");

	if (pDisplayViewLayout) {
		edje_object_signal_emit(_EDJ(pDisplayViewLayout), SIGNAL_MAIN_NOCONTENTS_HIDE, "*");
	}

	bIsDisplayViewOnlyAudio = FALSE;
}

void MpVideoDisplayViewPortrate(bool bLayoutHide, void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogError("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(bIsSpeedLongPress)
	{
		edje_object_part_text_set(_EDJ(pDisplayViewLayout), MAIN_SPEED_TEXT, "");
		edje_object_signal_emit(_EDJ(pDisplayViewLayout), SIGNAL_MAIN_SPEEDBOX_HIDE, "*");

		edje_object_signal_emit(pDisplayViewSpeedboxEdj, SIGNAL_SPEEDBOX_HIDE, "*");
		edje_object_part_unswallow(pDisplayViewCtrlEdj, pDisplayViewSpeedboxEdj);

		evas_object_hide(pDisplayViewSpeedboxEdj);
		bIsSpeedLongPress = FALSE;

		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedReset(pAppData);
	}

	elm_object_item_signal_emit(pNaviFrameItemForDisplayerView, SIGNAL_NAVIFRAME_PORTRATE_MODE, "*");
	edje_object_signal_emit(_EDJ(pDisplayViewLayout), SIGNAL_MAIN_PORTRATE_MODE, "*");
	edje_object_signal_emit(pDisplayViewPanelEdj, SIGNAL_PANEL_PORTRATE_MODE, "*");
	edje_object_signal_emit(pDisplayViewProgressbarEdj, SIGNAL_PROGRESS_PORTRATE_MODE, "*");
	edje_object_signal_emit(pDisplayViewCtrlEdj, SIGNAL_CONTROL_PORTRATE_MODE, "*");
	if(bLayoutHide)
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideLayout(pAppData);
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideFinish(pAppData);
	}
}

void MpVideoDisplayViewLandscape(bool bLayoutHide,void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogError("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(bIsSpeedLongPress)
	{
		edje_object_part_text_set(_EDJ(pDisplayViewLayout), MAIN_SPEED_TEXT, "");
		edje_object_signal_emit(_EDJ(pDisplayViewLayout), SIGNAL_MAIN_SPEEDBOX_HIDE, "*");

		edje_object_signal_emit(pDisplayViewSpeedboxEdj, SIGNAL_SPEEDBOX_HIDE, "*");
		edje_object_part_unswallow(pDisplayViewCtrlEdj, pDisplayViewSpeedboxEdj);

		evas_object_hide(pDisplayViewSpeedboxEdj);
		bIsSpeedLongPress = FALSE;

		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedReset(pAppData);
	}

	elm_object_item_signal_emit(pNaviFrameItemForDisplayerView, SIGNAL_NAVIFRAME_LANDSCAPE_MODE, "*");

	edje_object_signal_emit(_EDJ(pDisplayViewLayout), SIGNAL_MAIN_LANDSCAPE_MODE, "*");
	edje_object_signal_emit(pDisplayViewPanelEdj, SIGNAL_PANEL_LANDSCAPE_MODE, "*");
	edje_object_signal_emit(pDisplayViewProgressbarEdj, SIGNAL_PROGRESS_LANDSCAPE_MODE, "*");
	edje_object_signal_emit(pDisplayViewCtrlEdj, SIGNAL_CONTROL_LANDSCAPE_MODE, "*");

	if(bLayoutHide)
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideLayout(pAppData);
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideFinish(pAppData);
	}

}

void MpVideoDisplayViewResetProgressBar(void *pUserData)
{
	VideoLogInfo("");

	if (pDisplayViewLayout) {
		MpVideoProgressbarReset(pUserData);
	}
}

bool MpVideoDisplayViewIsPauseByUser(void)
{
	VideoLogInfo("%d", bIsPauseByUser);

	return bIsPauseByUser;
}

void MpVideoDisplayViewSetPauseByUser(bool bPauseByUser)
{
	VideoLogInfo("");

	bIsPauseByUser = bPauseByUser;
}


void MpVideoDisplayViewUpdateProgressBar(void *pUserData)
{
	VideoLogInfo("");

	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	if(bIsBeginOfStream)
	{
		MpVideoProgressbarCreateTimer(pUserData);
	}
}

void MpVideoDisplayViewSetMuteStatus(bool bMute)
{
	VideoLogInfo("");

	if(pDisplayViewCtrlEdj == NULL)
	{
		VideoLogError("Display control edj is NULL");
		return;
	}

	if(bMute)
	{
		edje_object_signal_emit(pDisplayViewCtrlEdj, SIGNAL_CONTROL_SET_MUTE, "*");
	}
	else
	{
		edje_object_signal_emit(pDisplayViewCtrlEdj, SIGNAL_CONTROL_UNSET_MUTE, "*");
	}

}

