/*
 * Copyright (c) [2012] Samsung Electronics Co., Ltd.
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <aul.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <syspopup_caller.h>

#include "mp-util.h"
#include "mp-video-log.h"
#include "video-player.h"
#include "mp-video-type-define.h"
#include "mp-video-value-define.h"
#include "mp-video-string-define.h"
#include "mp-video-progressbar.h"
#include "mp-video-animation-ctrl.h"
#include "mp-video-streaming-view.h"
#include "mp-video-service-ctrl.h"

#define CTRL_PANEL_SHOW_TIME					5.0

static Evas_Object *pNaviFrameHandle = NULL;
static Evas_Object *pStreamingViewNaviControlBar = NULL;
static Evas_Object *pBufferingAnimationHandle = NULL;

static Evas_Object *pStreamingViewLayout = NULL;
static Evas_Object *pStreamingViewPanelEdj = NULL;
static Evas_Object *pStreamingViewCtrlEdj = NULL;
static Evas_Object *pStreamingViewOptionEdj = NULL;
static Evas_Object *pStreamingViewProgressbarEdj = NULL;

static Ecore_Timer *pStreamingViewCtrlPanelHideTimer = NULL;
static Elm_Object_Item *pNaviFrameItemStreamingView = NULL;

static ChangeStreamingViewCbFunc pChangeViewUserCbFunc = NULL;

static bool bIsStreamingViewScreenLock = FALSE;
static bool bSwitchStreamingViewUI = TRUE;
static bool bIsStreamingHightQuality = TRUE;
static bool bIsChangeResolution = FALSE;
static bool bIsStreamingViewOnlyAudio = FALSE;
static VideoStreamingViewState nStreamingViewState = MP_STREAMING_VIEW_STATE_UNACTIVATE;

/*
 *	Callback function.
 */

static void MpVideoStreamingViewPressPauseButtonCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (!strcmp(szEmission, SIGNAL_CONTROL_PLAY_PAUSE_PRESS)) {
		VideoLogInfo("Press Pause button.");

		if (pAppData->nCurPlayerState == MP_PLAYER_STATE_RESUME || pAppData->nCurPlayerState == MP_PLAYER_STATE_PLAY) {
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause(pAppData);
		}
	}
}

static void MpVideoStreamingViewPressResumeButtonCb(void *pUserData,
						    Evas_Object *pObject,
						    const char *szEmission,
						    const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (!strcmp(szEmission, SIGNAL_CONTROL_PLAY_RESUME_PRESS)) {
		VideoLogInfo("Press Resume/Play button.");

		if (pAppData->nCurPlayerState == MP_PLAYER_STATE_PAUSE) {
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlResume(pAppData);
		}
	}
}

static void MpVideoStreamingViewBackButtonDownCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	edje_object_signal_callback_del(pStreamingViewCtrlEdj, SIGNAL_CONTROL_BACK_BUTTON_DOWN, SIGNAL_CONTROL_PART_BACK_BUTTON, (void *)MpVideoStreamingViewBackButtonDownCb);

	MpVideoStreamingViewPop(pNaviFrameHandle, pUserData);

	edje_object_signal_callback_add(pStreamingViewCtrlEdj, SIGNAL_CONTROL_BACK_BUTTON_DOWN, SIGNAL_CONTROL_PART_BACK_BUTTON, (void *)MpVideoStreamingViewBackButtonDownCb, pUserData);
}

static void MpVideoStreamingViewVolumeButtonDownCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogError("[ERR] No Exist pUserData.");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowVolume(pAppData);
}

static void MpVideoStreamingViewVolumeButtonUpCb(void *pUserData, Evas_Object *pObject, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");
	//syspopup_launch("volume", NULL);
}

static Eina_Bool MpVideoStreamingViewCtrlPanelHideCb(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR] No Exist pUserData.");
		return EINA_FALSE;
	}

	if(bIsStreamingViewOnlyAudio)
	{
		VideoLogInfo("Not hide control panel and UI because of audio only.");
		return EINA_FALSE;
	}

	if (MpVideoStreamingViewGetBufferingState()) {
		VideoLogInfo("skip for Buffering");
		return EINA_TRUE;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (pStreamingViewCtrlPanelHideTimer) {
		ecore_timer_del(pStreamingViewCtrlPanelHideTimer);
		pStreamingViewCtrlPanelHideTimer = NULL;
	}

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideLayout(pAppData);

	return EINA_FALSE;

}

static void MpVideoStreamingViewChangedCameraStatusCb(void *pNode, void *pUserData)
{
	if(!pUserData)
{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(!MpUtilIsCameraOff())
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}
		}

static void MpVideoStreamingViewHideLayoutFinish(void *pUserData, Evas_Object *pObj, const char *emission, const char *source)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogError("[ERR]");
		return;
		}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideFinish(pAppData);

	evas_object_hide(pStreamingViewLayout);
	}

static void MpVideoStreamingViewShowLayoutFinish(void *pUserData, Evas_Object *pObj, const char *emission, const char *source)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogError("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowFinish(pAppData);
}


/*
 * Internal function
 */

void MpVideoStreamingViewInitLayout(void *pParent, void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pStreamingViewLayout = elm_layout_add(pParent);
	if (pStreamingViewLayout) {
		Evas *pMainWindowEvas = evas_object_evas_get(pAppData->pMainWindow);

		elm_layout_file_set(pStreamingViewLayout, VIDEO_PLAYER_MAIN_EDJ, MAIN_EDJ_GROUP);

		// Option Button (Screen size, Share, Info, 5.1 Surround)
		pStreamingViewOptionEdj = MpUtilLoadEdjFile(pMainWindowEvas, VIDEO_PLAYER_OPTION_BUTTON_EDJ, OPTION_EDJ_GROUP);
		elm_object_part_content_set(pStreamingViewLayout, SWALLOW_MAIN_OPTION_BUTTON, pStreamingViewOptionEdj);
		edje_object_signal_emit(pStreamingViewOptionEdj, SIGNAL_OPTION_MODE_WEB, "*");

		pStreamingViewCtrlEdj = MpUtilLoadEdjFile(pMainWindowEvas, VIDEO_PLAYER_CONTROL_EDJ, CONTROL_EDJ_GROUP);
		pStreamingViewPanelEdj = MpUtilLoadEdjFile(pMainWindowEvas, VIDEO_PLAYER_PANEL_EDJ, PANEL_EDJ_GROUP);
		pBufferingAnimationHandle = MpVideoAnimationCtrlInit(pMainWindowEvas);

		edje_object_part_swallow(pStreamingViewPanelEdj, SWALLOW_PANEL_CONTROL, pStreamingViewCtrlEdj);
		elm_object_part_content_set(pStreamingViewLayout, SWALLOW_MAIN_ANIMATION, pBufferingAnimationHandle);
		edje_object_signal_emit(_EDJ(pStreamingViewLayout), SIGNAL_MAIN_ANIMATION_SHOW, "*");

		MpVideoAnimationCtrlShow();

		pStreamingViewProgressbarEdj = (Evas_Object *)MpVideoProgressbarInit(pAppData);
		MpVideoProgressbarSetSkipEvent(TRUE);
		if (pStreamingViewProgressbarEdj) {
			edje_object_part_swallow(pStreamingViewPanelEdj, SWALLOW_PANEL_PROGRESS_BAR, pStreamingViewProgressbarEdj);
		} else {
			VideoLogInfo("Progressbar edj handle is null.");
		}

		elm_object_part_content_set(pStreamingViewLayout, SWALLOW_MAIN_CONTROL, pStreamingViewPanelEdj);

		edje_object_signal_emit(pStreamingViewCtrlEdj, SIGNAL_CONTROL_MODE_STREAMING, "*");

		edje_object_signal_callback_add(pStreamingViewCtrlEdj,
					SIGNAL_CONTROL_PLAY_PAUSE_PRESS,
					SIGNAL_CONTROL_PART_PLAY_BUTTON,
					(void *)MpVideoStreamingViewPressPauseButtonCb, (void *)pAppData);

		edje_object_signal_callback_add(pStreamingViewCtrlEdj,
					SIGNAL_CONTROL_PLAY_RESUME_PRESS,
					SIGNAL_CONTROL_PART_PLAY_BUTTON,
					(void *)MpVideoStreamingViewPressResumeButtonCb, (void *)pAppData);

		edje_object_signal_callback_add(pStreamingViewCtrlEdj,
					SIGNAL_CONTROL_BACK_BUTTON_DOWN,
					SIGNAL_CONTROL_PART_BACK_BUTTON,
					(void *)MpVideoStreamingViewBackButtonDownCb, (void *)pAppData);

		edje_object_signal_callback_add(pStreamingViewCtrlEdj,
					SIGNAL_CONTROL_VOL_BUTTON_DOWN,
					SIGNAL_CONTROL_PART_VOL_BUTTON,
					(void *)MpVideoStreamingViewVolumeButtonDownCb, (void *)pAppData);

		edje_object_signal_callback_add(pStreamingViewCtrlEdj,
					SIGNAL_CONTROL_VOL_BUTTON_UP,
					SIGNAL_CONTROL_PART_VOL_BUTTON,
					(void *)MpVideoStreamingViewVolumeButtonUpCb, (void *)pAppData);

		edje_object_signal_callback_add(pStreamingViewCtrlEdj,
					SIGNAL_CONTROL_MUTE_BUTTON_DOWN,
					SIGNAL_CONTROL_PART_MUTE_BUTTON,
					(void *)MpVideoStreamingViewVolumeButtonDownCb, (void *)pAppData);

		edje_object_signal_callback_add(pStreamingViewCtrlEdj,
					SIGNAL_CONTROL_MUTE_BUTTON_UP,
					SIGNAL_CONTROL_PART_MUTE_BUTTON,
					(void *)MpVideoStreamingViewVolumeButtonUpCb, (void *)pAppData);

		MpUtilRegisteCameraStatusCbFunc((void *)MpVideoStreamingViewChangedCameraStatusCb, pUserData);
	}
}

void MpVideoStreamingViewInit(void *pUserData)
{
	VideoLogInfo("");

	if (!pNaviFrameHandle|| !pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	MpVideoStreamingViewInitLayout(pNaviFrameHandle, pUserData);
}



/*
 * External function
 */
void MpVideoStreamingViewPush(void *pNaviFrame, int nPlayerType, void *pUserData, ChangeStreamingViewCbFunc pChangeViewCb)
{
	VideoLogInfo("");

	pNaviFrameHandle = pNaviFrame;
	pChangeViewUserCbFunc = pChangeViewCb;

	MpVideoStreamingViewInit(pUserData);

	pNaviFrameItemStreamingView = elm_naviframe_item_push(pNaviFrameHandle,MP_STREAMING_PLAYER_STR, NULL, NULL, pStreamingViewLayout, "video/1line");

	elm_object_signal_callback_add(pStreamingViewLayout, SIGNAL_CONTORL_SHOW_FINISH, "*", MpVideoStreamingViewShowLayoutFinish, pUserData);
	elm_object_signal_callback_add(pStreamingViewLayout, SIGNAL_CONTORL_HIDE_FINISH, "*", MpVideoStreamingViewHideLayoutFinish, pUserData);

	if(MpUtilCheckMuteStatus())
	{
		MpVideoStreamingViewSetMuteStatus(TRUE);
	}

	int nRotate = MpUtilGetRotationState();
	if( nRotate == VIDEO_ROTATE_PORTRAIT_REVERSE || nRotate == VIDEO_ROTATE_PORTRAIT_NORMAL)
	{
		MpVideoStreamingViewPortrate();
	}
	else
	{
		MpVideoStreamingViewLandscape();
	}


}

void MpVideoStreamingViewPop(void *pNaviFrame, void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideVolume(pAppData);

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();
	if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}
	else
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}
}

void MpVideoStreamingViewDestroy(void *pUserData)
{
	VideoLogInfo("");

	MpVideoStreamingViewHideControlPanel(pUserData);
	MpVideoProgressbarDestroy();
	MpVideoAnimationCtrlDestroy();

	pNaviFrameHandle = NULL;
	pBufferingAnimationHandle = NULL;
	pNaviFrameItemStreamingView = NULL;
	pStreamingViewProgressbarEdj = NULL;

	if (pStreamingViewNaviControlBar) {
		evas_object_del(pStreamingViewNaviControlBar);
		pStreamingViewNaviControlBar = NULL;
	}

	if (pStreamingViewLayout) {
		evas_object_del(pStreamingViewLayout);
		pStreamingViewLayout = NULL;
	}

	if (pStreamingViewPanelEdj) {
		evas_object_del(pStreamingViewPanelEdj);
		pStreamingViewPanelEdj = NULL;
	}

	if (pStreamingViewCtrlEdj) {
		evas_object_del(pStreamingViewCtrlEdj);
		pStreamingViewCtrlEdj = NULL;
	}

	if (pStreamingViewCtrlPanelHideTimer) {
		ecore_timer_del(pStreamingViewCtrlPanelHideTimer);
		pStreamingViewCtrlPanelHideTimer = NULL;
	}

	if (pStreamingViewOptionEdj) {
		evas_object_del(pStreamingViewOptionEdj);
		pStreamingViewOptionEdj = NULL;
	}

	bIsStreamingViewScreenLock = FALSE;
	bIsStreamingViewOnlyAudio = FALSE;

	bSwitchStreamingViewUI = TRUE;
	nStreamingViewState = MP_STREAMING_VIEW_STATE_UNACTIVATE;

	bIsStreamingHightQuality = TRUE;
	bIsChangeResolution = FALSE;
}

void MpVideoStreamingViewInitAfterBeginOfStream(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;
	MpVideoProgressbarSetSkipEvent(FALSE);
	MpVideoProgressbarBeginOfStream((void *)pAppData);

	if (pAppData->nDuration > 1) {
		char szDurationTime[PLAY_TIME_LENGTH] = { 0 };
		snprintf(szDurationTime, PLAY_TIME_LENGTH, "%" PLAY_TIME_FORMAT,
			 PLAY_TIME_ARGS(pAppData->nDuration / 1000));
		MpVideoProgressbarUpdateDurationTime(szDurationTime);
	}

	MpVideoAnimationCtrlHide();
	edje_object_signal_emit(_EDJ(pStreamingViewLayout),
						SIGNAL_MAIN_ANIMATION_HIDE, "*");

	edje_object_signal_emit(pStreamingViewCtrlEdj, SIGNAL_CONTROL_PLAY_RESUME, "*");

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlReqDurationTime(pAppData);

	if(pAppData->nDuration < 1)
		pAppData->nDuration = 1;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowLayout(pAppData);
}

void MpVideoStreamingViewUpdatePauseKey(void)
{
	VideoLogInfo("");

	edje_object_signal_emit(pStreamingViewCtrlEdj, SIGNAL_CONTROL_PLAY_PAUSE, "*");
}

void MpVideoStreamingViewUpdateResumeKey(void)
{
	VideoLogInfo("");

	edje_object_signal_emit(pStreamingViewCtrlEdj, SIGNAL_CONTROL_PLAY_RESUME, "*");
}

void MpVideoStreamingViewShowControlPanel(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	if (!pStreamingViewPanelEdj) {
		VideoLogInfo("No exist play control panel edc.");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (nStreamingViewState == MP_STREAMING_VIEW_STATE_UNACTIVATE) {
		VideoLogInfo("");
		return;
	}

	bSwitchStreamingViewUI = TRUE;
	MpVideoProgressbarCreateTimer((void *)pAppData);

	evas_object_show(pStreamingViewLayout);

	elm_object_item_signal_emit(pNaviFrameItemStreamingView, SIGNAL_NAVIFRAME_TITLE_SHOW, "elm");

	int nRotate = MpUtilGetRotationState();
	if( nRotate == VIDEO_ROTATE_PORTRAIT_REVERSE || nRotate == VIDEO_ROTATE_PORTRAIT_NORMAL)
	{
		elm_object_signal_emit(pStreamingViewLayout, SIGNAL_MAIN_CONTROL_PORTRAIT_SHOW, "*");
	}
	else
	{
		elm_object_signal_emit(pStreamingViewLayout, SIGNAL_MAIN_CONTROL_LANDSCAPE_SHOW, "*");
	}

	if (pStreamingViewCtrlPanelHideTimer) {
		ecore_timer_del(pStreamingViewCtrlPanelHideTimer);
		pStreamingViewCtrlPanelHideTimer = NULL;
	}

	if (pAppData->nCurPlayerState == MP_PLAYER_STATE_PAUSE || bIsStreamingViewOnlyAudio == TRUE) {
	} else {
		pStreamingViewCtrlPanelHideTimer = ecore_timer_add(CTRL_PANEL_SHOW_TIME, MpVideoStreamingViewCtrlPanelHideCb, pUserData);
	}

	MpVideoStreamingViewActivate();
}

void MpVideoStreamingViewHideControlPanel(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	if (nStreamingViewState == MP_STREAMING_VIEW_STATE_UNACTIVATE) {
		VideoLogInfo("");
		return;
	}

	if (!pStreamingViewPanelEdj) {
		VideoLogInfo("No exist play control panel edc.");
		return;
	}

	if(bIsStreamingViewOnlyAudio)
	{
		VideoLogInfo("Not hide control panel and UI because of audio only.");
		return;
	}

	bSwitchStreamingViewUI = FALSE;
	MpVideoProgressbarDeleteTimer();

	elm_object_item_signal_emit(pNaviFrameItemStreamingView, SIGNAL_NAVIFRAME_TITLE_HIDE, "elm");
	elm_object_signal_emit(pStreamingViewLayout, SIGNAL_MAIN_CONTROL_NORMAL_HIDE, "*");

	if (pStreamingViewCtrlPanelHideTimer) {
		ecore_timer_del(pStreamingViewCtrlPanelHideTimer);
		pStreamingViewCtrlPanelHideTimer = NULL;
	}
}

void MpVideoStreamingViewActivate(void)
{
	VideoLogInfo("");

	nStreamingViewState = MP_STREAMING_VIEW_STATE_ACTIVATE;
}

void MpVideoStreamingViewUnactivate(void)
{
	VideoLogInfo("");

	nStreamingViewState = MP_STREAMING_VIEW_STATE_UNACTIVATE;
}

void MpVideoStreamingViewRotation(void *pUserData, int nRotateMode, bool bLayoutHide)
{
	VideoLogInfo("");
	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if (MpVideoStreamingViewGetBufferingState())
	{
		edje_object_signal_emit(_EDJ(pStreamingViewLayout), SIGNAL_MAIN_ANIMATION_SHOW, "*");
	}
	else
	{
		edje_object_signal_emit(_EDJ(pStreamingViewLayout), SIGNAL_MAIN_ANIMATION_HIDE, "*");
		if(bLayoutHide)
		{
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideLayout(pAppData);
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideFinish(pAppData);
		}
	}
}

void MpVideoStreamingViewSetBufferingRate(int nSetBufferingRate)
{
	if (nSetBufferingRate > 95) {
		MpVideoAnimationCtrlHide();
		MpVideoProgressbarSetSkipEvent(FALSE);
		edje_object_signal_emit(_EDJ(pStreamingViewLayout), SIGNAL_MAIN_ANIMATION_HIDE, "*");
	} else {
		MpVideoProgressbarSetSkipEvent(TRUE);
		MpVideoAnimationCtrlShow();
		edje_object_signal_emit(_EDJ(pStreamingViewLayout), SIGNAL_MAIN_ANIMATION_SHOW, "*");

		if(nSetBufferingRate >= 0)
			MpVideoAnimationCtrlSetPercent(nSetBufferingRate);
	}
}

bool MpVideoStreamingViewGetBufferingState(void)
{
	VideoLogInfo("");

	return MpVideoAnimationCtrlGetActiveState();
}

bool MpVideoStreamingViewIsTopView(void)
{
	VideoLogInfo("");

	Evas_Object *pTmpContents = NULL;
	Elm_Object_Item *pTmpItem = NULL;

	if (!pNaviFrameHandle) {
		VideoLogInfo("[ERR] No exist navigationbar handle.");
		return FALSE;
	}

	pTmpItem = elm_naviframe_top_item_get(pNaviFrameHandle);
	pTmpContents = elm_object_item_content_get(pTmpItem);
	if (pTmpContents) {
		if (pTmpContents == pStreamingViewLayout) {
			VideoLogInfo("Streaming display view is staing in top view.");
			return TRUE;
		} else {
			VideoLogInfo("Streaming display view is not staied in top view.");
			return FALSE;
		}
	} else {
		VideoLogInfo("[ERR] No exist naviFrame top item.");
		return FALSE;
	}

	return FALSE;
}

void MpVideoStreamingViewShowNocontentImage(void *pUserData)
{
	VideoLogInfo("");

	if (pStreamingViewLayout) {
		edje_object_signal_emit(_EDJ(pStreamingViewLayout), SIGNAL_MAIN_NOCONTENTS_SHOW, "*");
	}

	bIsStreamingViewOnlyAudio = TRUE;

	MpVideoStreamingViewShowControlPanel(pUserData);
}

void MpVideoStreamingViewHideNocontentImage(void)
{
	VideoLogInfo("");

	if (pStreamingViewLayout) {
		edje_object_signal_emit(_EDJ(pStreamingViewLayout), SIGNAL_MAIN_NOCONTENTS_HIDE, "*");
	}

	bIsStreamingViewOnlyAudio = FALSE;
}

void MpVideoStreamingViewPortrate(void)
{
	VideoLogInfo("");

	elm_object_item_signal_emit(pNaviFrameItemStreamingView, SIGNAL_NAVIFRAME_PORTRATE_MODE, "*");
	edje_object_signal_emit(_EDJ(pStreamingViewLayout), SIGNAL_MAIN_PORTRATE_MODE, "*");
	edje_object_signal_emit(pStreamingViewPanelEdj, SIGNAL_PANEL_PORTRATE_MODE, "*");
	edje_object_signal_emit(pStreamingViewProgressbarEdj, SIGNAL_PROGRESS_PORTRATE_MODE, "*");
	edje_object_signal_emit(pStreamingViewCtrlEdj, SIGNAL_CONTROL_STREAMING_PORTRATE_MODE, "*");
}

void MpVideoStreamingViewLandscape(void)
{
	VideoLogInfo("");

	elm_object_item_signal_emit(pNaviFrameItemStreamingView, SIGNAL_NAVIFRAME_LANDSCAPE_MODE, "*");
	edje_object_signal_emit(_EDJ(pStreamingViewLayout), SIGNAL_MAIN_LANDSCAPE_MODE, "*");
	edje_object_signal_emit(pStreamingViewPanelEdj, SIGNAL_PANEL_LANDSCAPE_MODE, "*");
	edje_object_signal_emit(pStreamingViewProgressbarEdj, SIGNAL_PROGRESS_LANDSCAPE_MODE, "*");
	edje_object_signal_emit(pStreamingViewCtrlEdj, SIGNAL_CONTROL_STREAMING_LANDSCAPE_MODE, "*");
}

void MpVideoStreamingViewUpdateProgressBar(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR]");
		return;
	}

	MpVideoProgressbarCreateTimer(pUserData);
}

void MpVideoStreamingViewSetMuteStatus(bool bMute)
{
	VideoLogInfo("");
	if(pStreamingViewCtrlEdj == NULL)
	{
		VideoLogError("streaming control edj is NULL");
		return;
	}

	if(bMute)
	{
		edje_object_signal_emit(pStreamingViewCtrlEdj, SIGNAL_CONTROL_SET_MUTE, "*");
	}
	else
	{
		edje_object_signal_emit(pStreamingViewCtrlEdj, SIGNAL_CONTROL_UNSET_MUTE, "*");
	}

}

