/*
 * Copyright (c) [2012] Samsung Electronics Co., Ltd.
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <vconf.h>
#include <vconf-keys.h>
#include <Ecore_Evas.h>
#include <Elementary.h>

#include "mp-util.h"
#include "mp-video-log.h"
#include "video-player.h"
#include "mp-video-service-ctrl.h"
#include "mp-video-subtitle-ctrl.h"

#define MP_VIDEO_SUBTITLE_MENU_NUM		2

#define MP_SUBTITLE_FONT_SMALL			22
#define MP_SUBTITLE_FONT_MEDIUM			35
#define MP_SUBTITLE_FONT_LARGE			48
#define MP_SUBTITLE_MIN_VALUE			3


/* subtitle edc layout */
static Evas_Object *pSubTitleEdj = NULL;

static bool bExistSubtitleFile = FALSE;
static char *szSubtitleFileUri = NULL;
static char szSubTitleExtWildkey[MP_SUBTITLE_TYPE_MAX][5] = { ".srt", ".sub", ".smi" };

static MpSubTitleSizeType nSubtitleSize = MP_SUBTITLE_SIZE_SMALL;

static bool bShowStatus = TRUE;


/*
 * Internal function
 */

static void MpVideoSubtitleCtrlClear(void)
{
	VideoLogInfo("");

	bExistSubtitleFile = FALSE;

	if(szSubtitleFileUri)
	{
		free(szSubtitleFileUri);
		szSubtitleFileUri = NULL;
	}

	if(pSubTitleEdj) {
		evas_object_del(pSubTitleEdj);
		pSubTitleEdj = NULL;
	}

}

/*
 * External function
 */

bool MpVideoSubtitleCtrlCheckSubtitleFile(char *szMediaUriPath)
{
	if (!szMediaUriPath) {
		VideoLogInfo("No Exist szMediaUri.");
		return FALSE;
	}

	if (strlen(szMediaUriPath) < 1) {
		VideoLogInfo("No Exist szMediaUri.");
		return FALSE;
	}

	VideoLogInfo("");

	struct stat buf;
	int nCount = 0;
	int nStrLength = 0;

	bExistSubtitleFile = FALSE;

	if(szSubtitleFileUri == NULL)
	{
		szSubtitleFileUri = calloc(1, sizeof(char) * STR_LEN_MAX);
	}

	char *szExt;

	szExt = strrchr(szMediaUriPath, '.');

	if (szExt) {
		nStrLength  = strlen(szMediaUriPath) - strlen(szExt);
	}
	else {
		nStrLength = strlen(szMediaUriPath);
	}

	for (nCount = 0; nCount < MP_SUBTITLE_TYPE_MAX; nCount++)
	{
		memset(szSubtitleFileUri, 0, STR_LEN_MAX);
		strncpy(szSubtitleFileUri, szMediaUriPath, nStrLength);
		strcat(szSubtitleFileUri, szSubTitleExtWildkey[nCount]);
		VideoLogInfo("%s",szSubtitleFileUri);

		if (!stat(szSubtitleFileUri, &buf)) {
			VideoLogInfo("Success finding subtitle file. - %d", nCount);
			bExistSubtitleFile = TRUE;
			break;
		} else {
			memset(szSubtitleFileUri, 0, STR_LEN_MAX);
		}
	}

	if (!bExistSubtitleFile) {
		VideoLogInfo("Fail finding subtitle file.");
	}

	return bExistSubtitleFile;
}

void *MpVideoSubtitleCtrlInit(void *pUserData, void *pParent)
{
	if (pUserData == NULL) {
		VideoLogInfo("[ERR]No have pUserData");
		return NULL;
	}

	if (pParent == NULL) {
		VideoLogInfo("[ERR]No have pParent");
		return NULL;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	MpVideoSubtitleCtrlClear();

	if(!MpVideoSubtitleCtrlCheckSubtitleFile(pAppData->szMediaUri)) 	{
		VideoLogInfo("Sub tiltle file is not exist.");
	}

	pSubTitleEdj = elm_layout_add(pParent);
	elm_layout_file_set(pSubTitleEdj, VIDEO_PLAYER_SUBTITLE_EDJ, SUBTITLE_EDJ_GROUP);

	return (void *)pSubTitleEdj;
}

void MpVideoSubtitleCtrlDestroy(void)
{
	VideoLogInfo("");

	MpVideoSubtitleCtrlClear();
}

void MpVideoSubtitleCtrlUpdate(char *szTxt)
{
	VideoLogInfo("%d",strlen(szTxt));

	if (!MpVideoSubtitleCtrlIsExistSubtitle()) {
		return;
	}

	if(pSubTitleEdj == NULL) {
		VideoLogInfo("Subtitle handle is NULL");
		return;
	}

	edje_object_part_text_set(_EDJ(pSubTitleEdj), "subtitle.label.txt",szTxt);
}


bool MpVideoSubtitleCtrlIsExistSubtitle(void)
{
	VideoLogInfo("");

	return bExistSubtitleFile;
}

bool MpVideoSubtitleCtrlGetSubtitleUri(char *szSubtitleUri)
{
	VideoLogInfo("");

	if (!MpVideoSubtitleCtrlIsExistSubtitle()) {
		return FALSE;
	}

	if (strlen(szSubtitleFileUri) < 1) {
		VideoLogInfo("[ERR]No have szSubtitleFileUri");
		return FALSE;
	}

	strncpy(szSubtitleUri, szSubtitleFileUri, STR_LEN_MAX -1);

	return TRUE;
}

void MpVideoSubtitleCtrlSetLayoutShow(bool bShow)
{
	VideoLogInfo("");

	if (!MpVideoSubtitleCtrlIsExistSubtitle()) {
		return;
	}

	if(pSubTitleEdj == NULL) {
		VideoLogInfo("Subtitle handle is NULL");
		return;
	}
	if(bShow) {
		edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_SUBTITLE_LAYOUT_SHOW, "*");
	}
	else {
		edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_SUBTITLE_LAYOUT_HIDE, "*");
	}

}

void MpVideoSubtitleCtrlSetSizeMode(int nSizeMode)
{
	VideoLogInfo("");

	if (!MpVideoSubtitleCtrlIsExistSubtitle()) {
		return;
	}

	if(pSubTitleEdj == NULL) {
		VideoLogInfo("Subtitle handle is NULL");
		return;
	}

	MpSubTitleSizeType type = nSizeMode;

	if(type == MP_SUBTITLE_SIZE_LARGE) {
		edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_SUBTITLE_SIZE_LARGE, "*");
	}
	else if(type == MP_SUBTITLE_SIZE_MEDIUM) {
		edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_SUBTITLE_SIZE_MEDIUM, "*");
	}
	else if(type == MP_SUBTITLE_SIZE_SMALL) {
		edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_SUBTITLE_SIZE_SMALL, "*");
	}

	nSubtitleSize = type;

}

int MpVideoSubtitleCtrlGetSizeMode()
{
	VideoLogInfo("");

	if (!MpVideoSubtitleCtrlIsExistSubtitle()) {
		return -1;
	}

	if(pSubTitleEdj == NULL) {
		VideoLogInfo("Subtitle handle is NULL");
		return -1;
	}

	return nSubtitleSize;
}



void MpVideoSubtitleCtrlSetRotate(int nRotateStatus)
{
	VideoLogInfo("");
	if (!MpVideoSubtitleCtrlIsExistSubtitle()) {
		return;
	}

	if(pSubTitleEdj == NULL) {
		VideoLogInfo("Subtitle handle is NULL");
		return;
	}

	if(nRotateStatus == VIDEO_ROTATE_PORTRAIT_NORMAL ||
		nRotateStatus == VIDEO_ROTATE_PORTRAIT_REVERSE) {
		edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_SUBTITLE_PORTRATE_MODE, "*");

	}
	else if(nRotateStatus == VIDEO_ROTATE_LANDSCAPE_NORMAL ||
		nRotateStatus == VIDEO_ROTATE_LANDSCAPE_REVERSE) {
		edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_SUBTITLE_LANDSCAPE_MODE, "*");
	}
}

void MpVideoSubtitleCtrlSetShowStatus(bool bShow)
{
	VideoLogInfo("");

	if(pSubTitleEdj == NULL) {
		VideoLogInfo("Subtitle handle is NULL");
		return;
	}
	if(bShowStatus != bShow) {
		bShowStatus = bShow;
		if(bShow) {
			edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_SUBTITLE_TXT_SHOW, "*");

		}
		else {
			edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_SUBTITLE_TXT_HIDE, "*");
		}
	}
}

bool MpVideoSubtitleCtrlGetShowStatus()
{
	VideoLogInfo("");

	if(pSubTitleEdj == NULL) {
		VideoLogInfo("Subtitle handle is NULL");
		return FALSE;
	}

	return bShowStatus;
}

void MpVideoSubtitleCtrlSetLockScreen(bool bShow)
{
	VideoLogInfo("");

	if(pSubTitleEdj == NULL) {
		VideoLogInfo("Subtitle handle is NULL");
		return;
	}

	if(bShow) {
		edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_LOCKSCREEN_SHOW, "*");
	}
	else {
		edje_object_signal_emit(_EDJ(pSubTitleEdj), SIGNAL_LOCKSCREEN_HIDE, "*");
	}

}

