/*
 * Copyright (c) [2012] Samsung Electronics Co., Ltd.
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */



#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <app.h>
#include <syspopup_caller.h>

#include "mp-util.h"
#include "mp-video-log.h"
#include "mp-external-ug.h"
#include "mp-video-view-mgr.h"
#include "mp-video-type-define.h"
#include "mp-util-media-service.h"
#include "mp-video-display-view.h"
#include "mp-video-string-define.h"
#include "mp-video-streaming-view.h"
#include "mp-video-service-ctrl.h"
#include "mp-video-volume-ctrl.h"

#define MULTIMEDIA_KEY_LONG_PRESS_INTERVAL		4.0
#define DETAIL_POPUP_STR_LEN_MAX				20
#define LAYOUT_VISIBLE_OFF_TIMER_INTERVAL		1.0


static Evas_Object *pMainLayout = NULL;
static Evas_Object *pBackGround = NULL;
static Evas_Object *pNaviFrame = NULL;
static Evas_Object *pConformant = NULL;
static Evas_Object *pGesture = NULL;
static Evas_Object *pEventRect = NULL;
static Evas_Object *pDisplayViewVolume = NULL;

static MpVideoRotateType nVideoRotateType = VIDEO_ROTATE_PORTRAIT_NORMAL;

static Ecore_Event_Handler *pHWKeyEventDownHandler = NULL;
static Ecore_Event_Handler *pHWKeyEventUpHandler = NULL;

static bool bIsLongPressForward = FALSE;
static bool bIsKeyCancle = FALSE;
static bool bShowLayout = FALSE;

static Ecore_Timer *pLongPressSpeedPlayingTimer = NULL;
static Ecore_Timer *pLongPressVolumeKeyTimer = NULL;
static Ecore_Timer *pLayoutVisibleOffTimer = NULL;

static void MpVideoVideoViewCtrlLongpressSpeedKeyUp(void *pUserData);
static void MpVideoVideoViewCtrlLongpressSpeedKeyDown(void *pUserData);
static void MpVideoViewCtrlChangeViewCb(int nChangePlayerView, void *pUserData);

/*
 * Callback function.
 */
static Eina_Bool MpVideoViewCtrlLayoutVisibleOffTimerCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return EINA_FALSE;
	}

	VideoLogInfo("");

	if(pLayoutVisibleOffTimer)
	{
		ecore_timer_del(pLayoutVisibleOffTimer);
		pLayoutVisibleOffTimer = NULL;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	MpUtilLayoutVisibleOff(pAppData->pMainWindow);

	return EINA_FALSE;
}

static Eina_Bool MpVideoViewCtrlVolumeKeyDownTimerCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return EINA_FALSE;
	}

	VideoLogInfo("");

	MpVideoViewCtrlShowVolume();
	int nVol;
	nVol = MpVideoVolumeCtrlGetVolume() - 1;
	MpVideoVolumeCtrlSetVolume(nVol);

	return EINA_TRUE;
}

static Eina_Bool MpVideoViewCtrlVolumeKeyUpTimerCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return EINA_FALSE;
	}

	VideoLogInfo("");

	MpVideoViewCtrlShowVolume();
	int nVol;
	nVol = MpVideoVolumeCtrlGetVolume() + 1;
	MpVideoVolumeCtrlSetVolume(nVol);

	return EINA_TRUE;
}


static Eina_Bool MpVideoViewCtrlHardkeyDownCb(void *pUserData, int nType, void *pEvent)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return EINA_FALSE;
	}

	VideoLogInfo("");

	Ecore_Event_Key *pKeyEvent = (Ecore_Event_Key *) pEvent;
	VideoAppData *pAppData = (VideoAppData *)pUserData;

	VideoLogInfo("Key down : %s", pKeyEvent->keyname);

	if(!strcmp(pKeyEvent->keyname, END_KEY))
	{
		VideoLogInfo("Back(End) key");
	}
	else if(!strcmp(pKeyEvent->keyname, HOME_KEY))
	{
		VideoLogInfo("Home key");
	}
	else if(!strcmp(pKeyEvent->keyname, VOLUME_DOWN_KEY))
				{
		MpVideoViewCtrlShowVolume();
		int nVol;
		nVol = MpVideoVolumeCtrlGetVolume() - 1;
		MpVideoVolumeCtrlSetVolume(nVol);

		if(pLongPressVolumeKeyTimer)
					{
			ecore_timer_del(pLongPressVolumeKeyTimer);
			pLongPressVolumeKeyTimer = NULL;
				}
		pLongPressVolumeKeyTimer = ecore_timer_add(0.3, MpVideoViewCtrlVolumeKeyDownTimerCb, (void *)pAppData);
					}
	else if(!strcmp(pKeyEvent->keyname, VOLUME_UP_KEY))
					{
		MpVideoViewCtrlShowVolume();
		int nVol;
		nVol = MpVideoVolumeCtrlGetVolume() + 1;
		MpVideoVolumeCtrlSetVolume(nVol);

		if(pLongPressVolumeKeyTimer)
				{
			ecore_timer_del(pLongPressVolumeKeyTimer);
			pLongPressVolumeKeyTimer = NULL;
				}

		pLongPressVolumeKeyTimer = ecore_timer_add(0.3, MpVideoViewCtrlVolumeKeyUpTimerCb, (void *)pAppData);
			}
	else if(!strcmp(pKeyEvent->keyname, POWER_KEY))
		{
		VideoLogInfo("Power key");

			MpUtilLcdOffForPowerButton();
		}
	else if(!strcmp(pKeyEvent->keyname, PAUSE_KEY))
	{
		VideoLogInfo("Pause key down");
	}
	else if(!strcmp(pKeyEvent->keyname, PLAY_KEY))
	{
		VideoLogInfo("Play key down");
	}
	else if(!strcmp(pKeyEvent->keyname, STOP_KEY))
	{
		VideoLogInfo("Stop key down");
	}
	else if(!strcmp(pKeyEvent->keyname, NEXT_KEY))
	{
		VideoLogInfo("Next key down");
	}
	else if(!strcmp(pKeyEvent->keyname, PREV_KEY))
	{
		VideoLogInfo("Prev key down");
	}
	else if(!strcmp(pKeyEvent->keyname, FORWARD_KEY))
	{
		VideoLogInfo("Forward key down");
		bIsLongPressForward = TRUE;
		MpVideoVideoViewCtrlLongpressSpeedKeyDown(pUserData);
	}
	else if(!strcmp(pKeyEvent->keyname, REWIND_KEY))
	{
		VideoLogInfo("Rewind key down");
		bIsLongPressForward = FALSE;
		MpVideoVideoViewCtrlLongpressSpeedKeyDown(pUserData);
	}
	else if(!strcmp(pKeyEvent->keyname, KEY_CANCLE))
	{
		VideoLogInfo("Cancel key down");
		bIsKeyCancle = TRUE;
	}

	return EINA_FALSE;
}

static Eina_Bool MpVideoViewCtrlHardkeyUpCb(void *pUserData, int nType, void *pEvent)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return EINA_FALSE;
	}

	VideoLogInfo("");

	Ecore_Event_Key *pKeyEvent = (Ecore_Event_Key *) pEvent;

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	VideoLogInfo("Key Up : %s", pKeyEvent->keyname);

	if(bIsKeyCancle)
	{
		VideoLogInfo("KEY CANCLE.");
		if(pLongPressVolumeKeyTimer)
		{
			ecore_timer_del(pLongPressVolumeKeyTimer);
			pLongPressVolumeKeyTimer = NULL;
		}
		bIsKeyCancle = FALSE;
		return EINA_FALSE;
	}

	if(!strcmp(pKeyEvent->keyname, END_KEY))
	{
		VideoLogInfo("End key up");
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}
	else if(!strcmp(pKeyEvent->keyname, VOLUME_DOWN_KEY))
	{
		if(pLongPressVolumeKeyTimer)
		{
			ecore_timer_del(pLongPressVolumeKeyTimer);
			pLongPressVolumeKeyTimer = NULL;
		}
	}
	else if(!strcmp(pKeyEvent->keyname, VOLUME_UP_KEY))
	{
		if(pLongPressVolumeKeyTimer)
		{
			ecore_timer_del(pLongPressVolumeKeyTimer);
			pLongPressVolumeKeyTimer = NULL;
		}
	}
	else if(!strcmp(pKeyEvent->keyname, HOME_KEY))
	{
		VideoLogInfo("Home key up");

		int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();
		if(nTmpPlayerType == MP_VIDEO_PLAYER ||
			nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
			nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
			nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
			nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
		{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}

		MpVideoServiceCtrlSetPrevPlayerType(MpVideoServiceCtrlGetPlayerType());
	}
	else if(!strcmp(pKeyEvent->keyname, POWER_KEY))
	{
		VideoLogInfo("Power key up");
	}
	else if(!strcmp(pKeyEvent->keyname, PAUSE_KEY))
	{
		VideoLogInfo("Pause key up");
		if(pAppData->nCurPlayerState == MP_PLAYER_STATE_PLAY || pAppData->nCurPlayerState == MP_PLAYER_STATE_RESUME)
		{
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause(pAppData);
		}
		else if(pAppData->nCurPlayerState == MP_PLAYER_STATE_PAUSE)
		{
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlResume(pAppData);
		}
	}
	else if(!strcmp(pKeyEvent->keyname, PLAY_KEY))
	{
		VideoLogInfo("Play key up");
		if(pAppData->nCurPlayerState == MP_PLAYER_STATE_PLAY || pAppData->nCurPlayerState == MP_PLAYER_STATE_RESUME)
		{
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause(pAppData);
		}
		else if(pAppData->nCurPlayerState == MP_PLAYER_STATE_PAUSE)
		{
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlResume(pAppData);
		}
	}
	else if(!strcmp(pKeyEvent->keyname, STOP_KEY))
	{
		VideoLogInfo("Stop key up");
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause(pAppData);
	}
	else if(!strcmp(pKeyEvent->keyname, NEXT_KEY))
	{
		VideoLogInfo("Next key up");
	}
	else if(!strcmp(pKeyEvent->keyname, PREV_KEY))
	{
		VideoLogInfo("Prev key up");
	}
	else if(!strcmp(pKeyEvent->keyname, FORWARD_KEY))
	{
		VideoLogInfo("Forward key up");
		MpVideoVideoViewCtrlLongpressSpeedKeyUp(pUserData);
	}
	else if(!strcmp(pKeyEvent->keyname, REWIND_KEY))
	{
		VideoLogInfo("Rewind key up");
		MpVideoVideoViewCtrlLongpressSpeedKeyUp(pUserData);
	}
	else
	{
	}

	return EINA_FALSE;
}

static Evas_Event_Flags MpVideoViewCtrlGestureNTabStartCb(void *pUserData , void *pEventInfo)
{
	VideoLogInfo("");

	return EVAS_EVENT_FLAG_NONE;
}


static Evas_Event_Flags MpVideoViewCtrlGestureNTabEndCb(void *pUserData , void *pEventInfo)
{
	VideoLogInfo("");

	Elm_Gesture_Taps_Info *p = (Elm_Gesture_Taps_Info *) pEventInfo;
	if(p->n == 1) {
		VideoAppData *pAppData = (VideoAppData *)pUserData;

		if(bShowLayout) {
			int nWidth = 0;
			int nHeight = 0;
			int nMinY = 0;
			int nMaxY = 0;

			evas_object_geometry_get(pMainLayout, NULL, NULL, &nWidth, &nHeight);

			if(MpUtilGetRotationState() == VIDEO_ROTATE_PORTRAIT_NORMAL ||
				MpUtilGetRotationState() == VIDEO_ROTATE_PORTRAIT_REVERSE)
			{
				nMinY = (elm_config_scale_get() * 223);
				nMaxY = nHeight - ( elm_config_scale_get() * 215);
			}
			else
			{
				nMinY = (elm_config_scale_get() * 223);
				nMaxY = nHeight - ( elm_config_scale_get() * 204);
			}

			if(p->y > nMinY && p->y < nMaxY)
			{
				pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideLayout(pAppData);
			}
		} else {
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlShowLayout(pAppData);
		}
	}

	return EVAS_EVENT_FLAG_NONE;
}


static Evas_Event_Flags MpVideoViewCtrlGestureNTabAbortCb(void *pUserData , void *pEventInfo)
{
	VideoLogInfo("");

	return EVAS_EVENT_FLAG_NONE;
}

/*
 * Internal function.
 */
static void MpVideoViewCtrlScreenRotationInit(int nRotateMode, void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR] No exist pUserData.");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	MpUtilRotateWindow(nRotateMode, pAppData->pMainWindow);
	nVideoRotateType = nRotateMode;
	MpVideoViewCtrlSetRotation(nVideoRotateType, FALSE, pUserData);
}

static Evas_Object *MpVideoViewCtrlCreateBg(Evas_Object *pWindow)
{
	if(!pWindow)
	{
		VideoLogError("[ERR]");
		return NULL;
	}

	VideoLogInfo("");

	return NULL;
}

void* MpVideoViewCtrlCreateConformant(void *pParent)
{
	if(!pParent)
	{
		VideoLogInfo("[ERR]");
		return NULL;
	}

	VideoLogInfo("");

	Evas_Object *pTmpConform = NULL;

	pTmpConform = elm_conformant_add(pParent);
	evas_object_size_hint_weight_set(pTmpConform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_conformant_set(pParent, EINA_TRUE);
	elm_win_resize_object_add(pParent, pTmpConform);
	evas_object_show(pTmpConform);

	pConformant = pTmpConform;

	return pTmpConform;
}

static Evas_Object *MpVideoViewCtrlCreateMainLayout(Evas_Object *pParent)
{
	if(!pParent)
	{
		VideoLogInfo("[ERR]");
		return NULL;
	}

	VideoLogInfo("");

	if(pMainLayout)
	{
		evas_object_del(pMainLayout);
		pMainLayout = NULL;
	}

	pMainLayout = elm_layout_add(pParent);

	elm_layout_theme_set(pMainLayout, "layout", "application", "video_no_indicator");
	evas_object_size_hint_weight_set(pMainLayout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_content_set(pParent, pMainLayout);
	pBackGround = MpVideoViewCtrlCreateBg(pParent);
	evas_object_show(pMainLayout);

	return pMainLayout;
}

static void MpVideoViewCtrlCreateEventRect(void *pUserData, Evas_Object *pParent)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	if(!pParent)
	{
		VideoLogInfo("[ERR]");
		return;
}

	VideoLogInfo("");

	pEventRect = evas_object_rectangle_add(evas_object_evas_get(pParent));
	evas_object_color_set(pEventRect, 0, 0, 0, 0);

	elm_object_part_content_set(pParent, "elm.swallow.event", pEventRect);

	pGesture = elm_gesture_layer_add(pEventRect);

	elm_gesture_layer_cb_set(pGesture, ELM_GESTURE_N_TAPS, ELM_GESTURE_STATE_START, MpVideoViewCtrlGestureNTabStartCb, pUserData);
	elm_gesture_layer_cb_set(pGesture, ELM_GESTURE_N_TAPS, ELM_GESTURE_STATE_END, MpVideoViewCtrlGestureNTabEndCb, pUserData);
	elm_gesture_layer_cb_set(pGesture, ELM_GESTURE_N_TAPS, ELM_GESTURE_STATE_ABORT, MpVideoViewCtrlGestureNTabAbortCb, pUserData);

	evas_object_repeat_events_set(pEventRect, EINA_TRUE);

	elm_gesture_layer_attach(pGesture, pEventRect);

	evas_object_show(pEventRect);


}
static Evas_Object *MpVideoViewCtrlCreateVolume(void *pUserData, Evas_Object *pParent)
{
	if(!pParent)
	{
		VideoLogInfo("[ERR]");
		return NULL;
	}

	VideoLogInfo("");

	if(pDisplayViewVolume)
	{
		evas_object_del(pDisplayViewVolume);
		pDisplayViewVolume = NULL;
	}

	pDisplayViewVolume = MpVideoVolumeCtrlInit(pParent, pUserData);

	return pDisplayViewVolume;
}

static void MpVideoViewCtrlShowNaviFrame(void)
{
	VideoLogInfo("");

	if(pNaviFrame)
	{
		elm_object_part_content_set(pMainLayout, "elm.swallow.content", pNaviFrame);
		evas_object_show(pNaviFrame);
	}
	}

static Evas_Object *MpVideoViewCtrlCreateNaviFrame(Evas_Object *pParent, void* pUserData)
{
	if(!pParent)
	{
		VideoLogInfo("[ERR]");
		return NULL;
	}

	VideoLogInfo("");

	if(pNaviFrame)
	{
		evas_object_del(pNaviFrame);
		pNaviFrame = NULL;
	}

	pNaviFrame = elm_naviframe_add(pParent);

	elm_object_part_content_set(pParent, "elm.swallow.content", pNaviFrame);

	elm_naviframe_prev_btn_auto_pushed_set(pNaviFrame, EINA_FALSE);

	evas_object_show(pNaviFrame);

	return pNaviFrame;
}

static void MpVideoViewCtrlSetVideoViews(VideoAppData *pAppData, int nPlayerType)
{
	VideoLogInfo("%p",pAppData);

	MpVideoServiceCtrlSetPlayerType(nPlayerType);

	switch(nPlayerType)
	{
	case MP_VIDEO_PLAYER:
		VideoLogInfo("MP_VIDEO_PLAYER");
		MpUtilMultimediaVolumeKeyGrab();
		MpVideoDisplayViewPush((void *)pNaviFrame, nPlayerType, (void *)pAppData, MpVideoViewCtrlChangeViewCb);
		break;

	case MP_VIDEO_PLAYER_SIMPLE:
		VideoLogInfo("MP_VIDEO_PLAYER_SIMPLE");
		MpUtilMultimediaVolumeKeyGrab();
		MpVideoDisplayViewPush((void *)pNaviFrame, nPlayerType, (void *)pAppData, MpVideoViewCtrlChangeViewCb);
		break;

	case MP_VIDEO_PLAYER_EMAIL:
		VideoLogInfo("MP_VIDEO_PLAYER_EMAIL");
		MpUtilMultimediaVolumeKeyGrab();
		MpVideoDisplayViewPush((void *)pNaviFrame, nPlayerType, (void *)pAppData, MpVideoViewCtrlChangeViewCb);
		break;

	case MP_VIDEO_PLAYER_MMS:
		VideoLogInfo("MP_VIDEO_PLAYER_MMS");
		MpUtilMultimediaVolumeKeyGrab();
		MpVideoDisplayViewPush((void *)pNaviFrame, nPlayerType, (void *)pAppData, MpVideoViewCtrlChangeViewCb);
		break;

	case MP_VIDEO_PLAYER_GALLERY:
		VideoLogInfo("MP_VIDEO_PLAYER_GALLERY");
		MpUtilMultimediaVolumeKeyGrab();
		MpVideoDisplayViewPush((void *)pNaviFrame, nPlayerType, (void *)pAppData, MpVideoViewCtrlChangeViewCb);
		break;

	case MP_STREAMING_PLAYER:
		VideoLogInfo("MP_STREAMING_PLAYER");
		MpUtilMultimediaVolumeKeyGrab();
		MpVideoStreamingViewPush((void *)pNaviFrame, nPlayerType, (void *)pAppData, MpVideoViewCtrlChangeViewCb);
		break;

	default:
		VideoLogInfo("[ERR] No exist player tpye.");
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}
}

static Eina_Bool MpVideoVideoViewCtrlSpeedKeyLongPressCb(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return EINA_FALSE;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(pAppData->nPlayingSpeed > MP_PLAYING_SPEED_MAX)
	{
		if(pLongPressSpeedPlayingTimer)
		{
			ecore_timer_del(pLongPressSpeedPlayingTimer);
			pLongPressSpeedPlayingTimer = NULL;
		}

		return EINA_FALSE;
	}

	++pAppData->nPlayingSpeed;

	if(bIsLongPressForward)
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedFF(pAppData);
	}
	else
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedRew(pAppData);
	}

	return EINA_TRUE;
}

static void MpVideoVideoViewCtrlLongpressSpeedKeyDown(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist pUserData.");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->nPlayingSpeed = MP_PLAYING_SPEED_2X;

	if(bIsLongPressForward)
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedFF(pAppData);
	}
	else
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedRew(pAppData);
	}

	if(pLongPressSpeedPlayingTimer)
	{
		ecore_timer_del(pLongPressSpeedPlayingTimer);
		pLongPressSpeedPlayingTimer = NULL;
	}

	pLongPressSpeedPlayingTimer = ecore_timer_add(MULTIMEDIA_KEY_LONG_PRESS_INTERVAL, MpVideoVideoViewCtrlSpeedKeyLongPressCb, (void *)pAppData);
}

static void MpVideoVideoViewCtrlLongpressSpeedKeyUp(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist pUserData.");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	bIsLongPressForward = FALSE;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlSpeedReset(pAppData);

	if(pLongPressSpeedPlayingTimer)
	{
		ecore_timer_del(pLongPressSpeedPlayingTimer);
		pLongPressSpeedPlayingTimer = NULL;
	}
}

static void MpVideoViewCtrlChangeViewCb(int nChangePlayerView, void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	MpVideoViewCtrlHideVolume();

	MpVideoViewCtrlSetVideoViews((VideoAppData*)pUserData, nChangePlayerView);

		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlReLaunching(pAppData);
	}

/*
 * External function.
 */
bool MpVideoViewCtrlInit(VideoAppData *pAppData)
{
	if(!pAppData)
	{
		VideoLogInfo("[ERR]");
		return FALSE;
	}

	VideoLogInfo("");

	if(pHWKeyEventDownHandler)
	{
		ecore_event_handler_del(pHWKeyEventDownHandler);
		pHWKeyEventDownHandler = NULL;
	}

	if(pHWKeyEventUpHandler)
	{
		ecore_event_handler_del(pHWKeyEventUpHandler);
		pHWKeyEventUpHandler = NULL;
	}

	pHWKeyEventDownHandler = ecore_event_handler_add(ECORE_EVENT_KEY_DOWN, (void *)MpVideoViewCtrlHardkeyDownCb, (void *)pAppData);
	pHWKeyEventUpHandler = ecore_event_handler_add(ECORE_EVENT_KEY_UP, (void *)MpVideoViewCtrlHardkeyUpCb, (void *)pAppData);

	elm_win_indicator_mode_set(pAppData->pMainWindow, ELM_WIN_INDICATOR_SHOW);

	MpVideoViewCtrlCreateConformant(pAppData->pMainWindow);
	MpVideoViewCtrlCreateMainLayout(pConformant);
	MpVideoViewCtrlCreateEventRect((void *)pAppData, pMainLayout);
	MpVideoViewCtrlCreateVolume(pAppData, pMainLayout);
	MpVideoViewCtrlCreateNaviFrame(pMainLayout, (void*)pAppData);
	MpVideoViewCtrlSetVideoViews(pAppData, MpVideoServiceCtrlGetPlayerType());
	MpVideoViewCtrlScreenRotationInit(MpUtilGetRotationState(), (void *)pAppData);

	return TRUE;
}

void MpVideoViewCtrlDestroy(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	MpExternalUgUnload();

	MpVideoVolumeCtrlDestroy();

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		MpVideoDisplayViewDestroy(pUserData);
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoStreamingViewDestroy(pUserData);
	}

	if(pGesture)
	{
		evas_object_del(pGesture);
		pGesture = NULL;
	}

	if(pEventRect)
	{
		evas_object_del(pEventRect);
		pEventRect = NULL;
	}

	if(pHWKeyEventDownHandler)
	{
		ecore_event_handler_del(pHWKeyEventDownHandler);
		pHWKeyEventDownHandler = NULL;
	}

	if(pHWKeyEventUpHandler)
	{
		ecore_event_handler_del(pHWKeyEventUpHandler);
		pHWKeyEventUpHandler = NULL;
	}

	if(pMainLayout)
	{
		evas_object_del(pMainLayout);
		pMainLayout = NULL;
	}

	if(pLongPressSpeedPlayingTimer)
	{
		ecore_timer_del(pLongPressSpeedPlayingTimer);
		pLongPressSpeedPlayingTimer = NULL;
	}

	if(pLongPressVolumeKeyTimer)
	{
		ecore_timer_del(pLongPressVolumeKeyTimer);
		pLongPressVolumeKeyTimer = NULL;
	}

	if(pLayoutVisibleOffTimer)
	{
		ecore_timer_del(pLayoutVisibleOffTimer);
		pLayoutVisibleOffTimer = NULL;
	}

	if(pDisplayViewVolume)
	{
		evas_object_del(pDisplayViewVolume);
		pDisplayViewVolume = NULL;
	}

	if(pBackGround)
	{
		evas_object_del(pBackGround);
		pBackGround = NULL;
	}

	bIsLongPressForward = FALSE;
	bIsKeyCancle = FALSE;

	nVideoRotateType = VIDEO_ROTATE_PORTRAIT_NORMAL;
}

void MpVideoViewCtrlSetRotation(int nRotateMode, bool bLayoutHide, void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR] No exist pUserData.");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(nRotateMode == VIDEO_ROTATE_UNKNOWN)
	{
		nRotateMode = MpUtilGetRotationState();
	}

	nVideoRotateType = nRotateMode;

	MpUtilSetRotationState(nVideoRotateType);

	if(MpVideoVolumeCtrlGetShowStatus())
	{
		MpVideoViewCtrlHideVolume();
	}

	if(nVideoRotateType == VIDEO_ROTATE_PORTRAIT_NORMAL || nVideoRotateType == VIDEO_ROTATE_PORTRAIT_REVERSE)
	{
		edje_object_signal_emit(_EDJ(pMainLayout), SIGNAL_VOLUME_PORTRAIT_MODE, "*");
	}
	else if(nVideoRotateType == VIDEO_ROTATE_LANDSCAPE_NORMAL || nVideoRotateType == VIDEO_ROTATE_LANDSCAPE_REVERSE)
	{
		edje_object_signal_emit(_EDJ(pMainLayout), SIGNAL_VOLUME_LANDSCAPE_MODE, "*");
	}

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		if(nVideoRotateType == VIDEO_ROTATE_PORTRAIT_NORMAL || nVideoRotateType == VIDEO_ROTATE_PORTRAIT_REVERSE)
		{
			MpVideoDisplayViewPortrate(bLayoutHide, pUserData);
		}
		else if(nVideoRotateType == VIDEO_ROTATE_LANDSCAPE_NORMAL || nVideoRotateType == VIDEO_ROTATE_LANDSCAPE_REVERSE)
		{
			MpVideoDisplayViewLandscape(bLayoutHide, pUserData);
		}
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoStreamingViewRotation((void *)pAppData, nVideoRotateType, bLayoutHide);
	}

	MpUtilRotateWindow(nVideoRotateType, pAppData->pMainWindow);

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlRotateScreen(pAppData, nVideoRotateType);
}

void MpVideoViewCtrlInitAfterBeginOfStream(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		MpVideoDisplayViewInitAfterBeginOfStream(pUserData);
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoStreamingViewInitAfterBeginOfStream(pUserData);
	}
}

void MpVideoViewCtrlPause(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		MpVideoDisplayViewUpdatePauseKey();
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoStreamingViewUpdatePauseKey();
	}
}

void MpVideoViewCtrlResume(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		MpVideoDisplayViewUpdateResumeKey();
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoStreamingViewUpdateResumeKey();
	}
}

void MpVideoViewShowLayout(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(pLayoutVisibleOffTimer)
	{
		ecore_timer_del(pLayoutVisibleOffTimer);
		pLayoutVisibleOffTimer = NULL;
	}

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		edje_object_signal_emit(_EDJ(pMainLayout), SIGNAL_LAYOUT_INDICATOR_SHOW, "*");
			elm_win_indicator_mode_set(pAppData->pMainWindow, ELM_WIN_INDICATOR_SHOW);
			MpVideoViewCtrlShowNaviFrame();
			bShowLayout = TRUE;

			MpVideoDisplayViewActivate();
			MpVideoDisplayViewShowControlPanel(pUserData);
		}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		edje_object_signal_emit(_EDJ(pMainLayout), SIGNAL_LAYOUT_INDICATOR_SHOW, "*");
			elm_win_indicator_mode_set(pAppData->pMainWindow, ELM_WIN_INDICATOR_SHOW);
			MpVideoViewCtrlShowNaviFrame();
			bShowLayout = TRUE;

		MpVideoStreamingViewActivate();
		MpVideoStreamingViewShowControlPanel(pUserData);
	}
}

void MpVideoViewHideLayout(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		if(MpVideoStreamingViewGetBufferingState())
		{
			VideoLogInfo("skip for Buffering");
			return;
		}
	}

	edje_object_signal_emit(_EDJ(pMainLayout), SIGNAL_LAYOUT_INDICATOR_HIDE, "*");
	elm_win_indicator_mode_set(pAppData->pMainWindow, ELM_WIN_INDICATOR_HIDE);

	bShowLayout = FALSE;

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		MpVideoDisplayViewHideControlPanel(pUserData);
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoStreamingViewHideControlPanel(pUserData);
	}
}
void MpVideoViewCtrlShowFinish(void *pUserData)
{
	VideoLogInfo("");

}

void MpVideoViewCtrlHideFinish(void *pUserData, bool bLayoutOff)
{
	if(!pUserData)
	{
		VideoLogError("[ERR]");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(pNaviFrame)
	{
		elm_object_part_content_unset(pMainLayout, "elm.swallow.content");
		evas_object_hide(pNaviFrame);
	}

	if(pLayoutVisibleOffTimer)
	{
		ecore_timer_del(pLayoutVisibleOffTimer);
		pLayoutVisibleOffTimer = NULL;
	}

	if(bLayoutOff)
	{
		pLayoutVisibleOffTimer = ecore_timer_add(LAYOUT_VISIBLE_OFF_TIMER_INTERVAL, MpVideoViewCtrlLayoutVisibleOffTimerCb, (void *)pAppData);
	}


}

void MpVideoViewSetBufferingRateForStreamingView(int nSetBufferingRate, void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

//	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerTypeWithoutLog();

	if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		if(MpUtilCheckRTSPType(pAppData->szMediaUri))
		{
			if(nSetBufferingRate == 0)
			{
				MpVideoStreamingViewSetBufferingRate(-1);
			}
			else
			{
				MpVideoStreamingViewSetBufferingRate(nSetBufferingRate);
			}
		}
		else
		{
			if(nSetBufferingRate >= 0)
			{
				MpVideoStreamingViewSetBufferingRate(nSetBufferingRate);
			}
		}
	}
}

void MpVideoViewCtrlChangeTitle(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		VideoAppData *pAppData = (VideoAppData *)pUserData;

		memset(pAppData->szSubtitle, 0, sizeof(char) * STR_LEN_MAX);

		MpVideoDisplayViewChangeNaviFrameTitle(pUserData);
	}
}

bool MpVideoViewCtrlIsTopView(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return FALSE;
	}

	VideoLogInfo("");

	bool bIstopView = FALSE;

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		VideoLogInfo("VIDEO PLAYER");
		bIstopView = MpVideoDisplayViewIsTopView();
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		VideoLogInfo("STREAMING PLAYER");
		bIstopView = MpVideoStreamingViewIsTopView();
	}

	return bIstopView;
}

void MpVideoViewCtrlShowNocontentImage(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		MpVideoDisplayViewShowNocontentImage(pUserData);
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoStreamingViewShowNocontentImage(pUserData);
	}
}

void MpVideoViewCtrlHideNocontentImage(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("[ERR]");
		return;
	}

	VideoLogInfo("");

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		MpVideoDisplayViewHideNocontentImage();
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoStreamingViewHideNocontentImage();
	}
}

void MpVideoViewCtrlResetProgressBar(void *pUserData)
{
	VideoLogInfo("");

	MpVideoDisplayViewResetProgressBar(pUserData);
}

void MpVideoViewCtrlUpdateProgressbar(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist pUserData.");
		return;
	}
	VideoLogInfo("");
	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		VideoLogInfo("VIDEO PLAYER");
		MpVideoDisplayViewUpdateProgressBar(pUserData);
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		VideoLogInfo("STREAMING PLAYER");
		MpVideoStreamingViewUpdateProgressBar(pUserData);
	}
}

bool MpVideoViewCtrlIsPauseByUser(void *pUserData)
{
	if(!pUserData)
{
		VideoLogInfo("[ERR]");
		return FALSE;
	}

	VideoLogInfo("");

	return MpVideoDisplayViewIsPauseByUser();
}

void MpVideoViewCtrlSetMuteValue(bool bMute)
{

	VideoLogInfo("");
	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
		nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
		nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
		nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
		nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		MpVideoDisplayViewSetMuteStatus(bMute);
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		MpVideoStreamingViewSetMuteStatus(bMute);
	}
}


void MpVideoViewCtrlShowVolume()
{
	VideoLogInfo("");

	if(MpVideoVolumeCtrlGetShowStatus())
	{
		return;
	}

	elm_object_part_content_set(pMainLayout, "elm.swallow.volume", pDisplayViewVolume);

	MpVideoVolumeCtrlShow();

}
void MpVideoViewCtrlHideVolume()
{
	VideoLogInfo("");

	if(pLongPressVolumeKeyTimer)
	{
		ecore_timer_del(pLongPressVolumeKeyTimer);
		pLongPressVolumeKeyTimer = NULL;
	}

	elm_object_part_content_unset(pMainLayout, "elm.swallow.volume");

	MpVideoVolumeCtrlHide();

}
