/*
 * Copyright (c) [2012] Samsung Electronics Co., Ltd.
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <sound_manager.h>
#include <Elementary.h>

#include "video-player.h"

#include "mp-util.h"
#include "mp-video-log.h"
#include "mp-video-type-define.h"
#include "mp-video-value-define.h"
#include "mp-video-string-define.h"
#include "mp-video-volume-ctrl.h"

//#include "mp-video-statistics.h"

#define VOLUME_CTRL_HIDE_TIMER_INTERVAL		3.0
#define VOLUME_CTRL_DRAG_TIMER_INTERVAL		0.3


typedef struct __MpVolumeCtrlData MpVolumeCtrlData;

struct __MpVolumeCtrlData{
	VideoAppData 		*pAppData;

	Evas_Object 		*pLayout;

	Ecore_Timer		*pHideTimer;
	Ecore_Timer		*pDragTimer;

	int			nMaxVol;
	int			nMinVol;

	int 			nCurVol;

};

static MpVolumeCtrlData	*pVolumeData = NULL;

static void MpVideoVolumeCtrlUpdateTextValue(void *pUserData)
{

	VideoLogInfo("");
	if(pUserData == NULL)
	{
		VideoLogError("Volume ctrl data is null.");
		return;
	}

	VideoLogInfo("");

	MpVolumeCtrlData	*pCtrlData = (MpVolumeCtrlData *)pUserData;

	char szVol[3] = {0,};
	snprintf(szVol, 3, "%d", pCtrlData->nCurVol);

	VideoLogError("%d , %s", pCtrlData->nCurVol, szVol);

	edje_object_part_text_set(_EDJ(pCtrlData->pLayout), "volume.progress.circle.txt", szVol);

}

static void MpVideoVolumeCtrlUpdateDragValue(void *pUserData)
{

	VideoLogInfo("");
	if(pUserData == NULL)
	{
		VideoLogError("Volume ctrl data is null.");
		return;
	}

	VideoLogInfo("");

	MpVolumeCtrlData	*pCtrlData = (MpVolumeCtrlData *)pUserData;


	double fVolPos = 0.0;
	fVolPos = ((double)pCtrlData->nCurVol/(double)pCtrlData->nMaxVol);
	edje_object_part_drag_value_set(_EDJ(pCtrlData->pLayout), "volume.progress.drag", fVolPos, 0.0);

}

static int MpVideoVolumeCtrlGetMaxVolume()
{
	VideoLogInfo("");

	int nRet = SOUND_MANAGER_ERROR_NONE;
	int nMax = 0;
	nRet = sound_manager_get_max_volume (SOUND_TYPE_MEDIA, &nMax);
	if(nRet != SOUND_MANAGER_ERROR_NONE)
	{
		VideoLogError("Sound manage get max volue fail : %d", nRet);
		return -1;
	}

	return nMax;
}
static int MpVideoVolumeCtrlGetCurVolume()
{
	VideoLogInfo("");

	int nRet = SOUND_MANAGER_ERROR_NONE;
	int nVol = 0;
	nRet = sound_manager_get_volume  (SOUND_TYPE_MEDIA, &nVol);
	if(nRet != SOUND_MANAGER_ERROR_NONE)
	{
		VideoLogError("Sound manage get current volue fail : %d", nRet);
		return -1;
	}

	return nVol;
}

static void MpVideoVolumeCtrlSetCurVolume(int nVol)
{
	VideoLogInfo("");

	int nRet = SOUND_MANAGER_ERROR_NONE;
	nRet = sound_manager_set_volume  (SOUND_TYPE_MEDIA, nVol);
	if(nRet != SOUND_MANAGER_ERROR_NONE)
	{
		VideoLogError("Sound manage get current volue fail : %d", nRet);
		return;
	}

	return;
}

static Eina_Bool MpVideoVolumeCtrlHideTimerCb(void *pUserData)
{
	VideoLogInfo("");
	if(pUserData == NULL)
	{
		VideoLogError("pAppData is null.");
		return EINA_FALSE;
	}

	MpVolumeCtrlData	*pCtrlData = (MpVolumeCtrlData *)pUserData;
	if(pCtrlData->pHideTimer)
	{
		ecore_timer_del(pCtrlData->pHideTimer);
		pCtrlData->pHideTimer = NULL;
	}

	pCtrlData->pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideVolume(pCtrlData->pAppData);

	return EINA_FALSE;
}


static Eina_Bool MpVideoVolumeCtrlDragTimerCb(void *pUserData)
{
	VideoLogInfo("");
	if(pUserData == NULL)
	{
		VideoLogError("pAppData is null.");
		return EINA_FALSE;
	}

	VideoLogInfo("");

	MpVolumeCtrlData	*pCtrlData = (MpVolumeCtrlData *)pUserData;

	double fVolPos = 0.0;

	edje_object_part_drag_value_get(_EDJ(pCtrlData->pLayout), "volume.progress.drag", &fVolPos, NULL);

	pCtrlData->nCurVol = (int)(pCtrlData->nMaxVol * fVolPos);

	MpVideoVolumeCtrlSetCurVolume(pCtrlData->nCurVol);

	MpVideoVolumeCtrlUpdateTextValue((void *)pCtrlData);

	return EINA_TRUE;
}



static void MpVideoVolumeCtrlDragStartcb(void *pUserData, Evas_Object *obj, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");
	if(pUserData == NULL)
	{
		VideoLogError("pAppData is null.");
		return;
	}
	MpVolumeCtrlData	*pCtrlData = (MpVolumeCtrlData *)pUserData;

	if(pCtrlData->pHideTimer)
	{
		ecore_timer_del(pCtrlData->pHideTimer);
		pCtrlData->pHideTimer = NULL;
	}

	pCtrlData->pDragTimer = ecore_timer_add(VOLUME_CTRL_DRAG_TIMER_INTERVAL, MpVideoVolumeCtrlDragTimerCb, (void *)pCtrlData);
}

static void MpVideoVolumeCtrlDragStopcb(void *pUserData, Evas_Object *obj, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");
	if(pUserData == NULL)
	{
		VideoLogError("pAppData is null.");
		return;
	}
	MpVolumeCtrlData	*pCtrlData = (MpVolumeCtrlData *)pUserData;

	if(pCtrlData->pHideTimer)
	{
		ecore_timer_del(pCtrlData->pHideTimer);
		pCtrlData->pHideTimer = NULL;
	}

	pCtrlData->pHideTimer = ecore_timer_add(VOLUME_CTRL_HIDE_TIMER_INTERVAL, MpVideoVolumeCtrlHideTimerCb, (void *)pCtrlData);

	if(pCtrlData->pDragTimer)
	{
		ecore_timer_del(pCtrlData->pDragTimer);
		pCtrlData->pDragTimer = NULL;
	}

	double fVolPos = 0.0;
	edje_object_part_drag_value_get(_EDJ(pCtrlData->pLayout), "volume.progress.drag", &fVolPos, NULL);
	pCtrlData->nCurVol = (int)(pCtrlData->nMaxVol * fVolPos);

	MpVideoVolumeCtrlSetCurVolume(pCtrlData->nCurVol);
	MpVideoVolumeCtrlUpdateTextValue((void *)pCtrlData);
}

static void MpVideoVolumeCtrlDragcb(void *pUserData, Evas_Object *obj, const char *szEmission, const char *szSource)
{
	VideoLogInfo("");

	if(pUserData == NULL)
	{
		VideoLogError("pAppData is null.");
		return;
	}

	VideoLogInfo("");

	MpVolumeCtrlData	*pCtrlData = (MpVolumeCtrlData *)pUserData;
	double fVolPos = 0.0;
	edje_object_part_drag_value_get(_EDJ(pCtrlData->pLayout), "volume.progress.drag", &fVolPos, NULL);
	pCtrlData->nCurVol = (int)(pCtrlData->nMaxVol * fVolPos);

	MpVideoVolumeCtrlUpdateTextValue((void *)pCtrlData);

}


void *MpVideoVolumeCtrlInit(void *pParent, void* pUserData)
{
	if(pUserData == NULL)
	{
		VideoLogError("pAppData is null.");
		return NULL;
	}

	if(pParent == NULL)
	{
		VideoLogError("pParent is null.");
		return NULL;
	}

	VideoLogInfo("");

	MpVolumeCtrlData	*pCtrlData = NULL;
	VideoAppData 		*pAppData = (VideoAppData *)pUserData;

	pCtrlData = calloc(1, sizeof(MpVolumeCtrlData));

	pCtrlData->nMinVol = 0;
	pCtrlData->nMaxVol = MpVideoVolumeCtrlGetMaxVolume();
	pCtrlData->nCurVol = MpVideoVolumeCtrlGetCurVolume();

	Evas_Object *layout = elm_layout_add(pParent);
	elm_layout_file_set(layout, VIDEO_PLAYER_VOLUME_POPUP_EDJ, VOLUME_EDJ_GROUP);
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_hide(layout);

	edje_object_signal_callback_add(_EDJ(layout), "drag,start", "volume.progress.drag", MpVideoVolumeCtrlDragStartcb, (void *)pCtrlData);
	edje_object_signal_callback_add(_EDJ(layout), "drag,stop", "volume.progress.drag", MpVideoVolumeCtrlDragStopcb, (void *)pCtrlData);
	edje_object_signal_callback_add(_EDJ(layout), "drag", "volume.progress.drag", MpVideoVolumeCtrlDragcb, (void *)pCtrlData);

	pCtrlData->pAppData = pAppData;
	pCtrlData->pLayout = layout;

	MpVideoVolumeCtrlSetCurVolume(pCtrlData->nCurVol);
	MpVideoVolumeCtrlUpdateTextValue((void *)pCtrlData);
	MpVideoVolumeCtrlUpdateDragValue((void *)pCtrlData);

	pVolumeData = pCtrlData;

	return layout;

}

void MpVideoVolumeCtrlDestroy(void)
{
	VideoLogInfo("");

	if(pVolumeData == NULL)
	{
		VideoLogError("Volume Ctrl data is NULL");
		return;
	}

	if(pVolumeData->pLayout)
	{
		evas_object_del(pVolumeData->pLayout);
		pVolumeData->pLayout = NULL;

	}

	if(pVolumeData->pHideTimer)
	{
		ecore_timer_del(pVolumeData->pHideTimer);
		pVolumeData->pHideTimer = NULL;
	}

	if(pVolumeData->pDragTimer)
	{
		ecore_timer_del(pVolumeData->pDragTimer);
		pVolumeData->pDragTimer = NULL;
	}

	free(pVolumeData);
	pVolumeData = NULL;

}


void MpVideoVolumeCtrlShow()
{
	VideoLogInfo("");

	if(pVolumeData == NULL)
	{
		VideoLogError("Volume Ctrl data is NULL");
		return;
	}
	MpVolumeCtrlData *pCtrlData = pVolumeData;

	evas_object_show(pCtrlData->pLayout);

	if(pCtrlData->pHideTimer)
	{
		ecore_timer_del(pCtrlData->pHideTimer);
		pCtrlData->pHideTimer = NULL;
	}

	pCtrlData->pHideTimer = ecore_timer_add(VOLUME_CTRL_HIDE_TIMER_INTERVAL, MpVideoVolumeCtrlHideTimerCb, (void *)pCtrlData);

}

void MpVideoVolumeCtrlHide()
{
	VideoLogInfo("");

	if(pVolumeData == NULL)
	{
		VideoLogError("Volume Ctrl data is NULL");
		return;
	}

	MpVolumeCtrlData *pCtrlData = pVolumeData;

	if(pCtrlData->pHideTimer)
	{
		ecore_timer_del(pCtrlData->pHideTimer);
		pCtrlData->pHideTimer = NULL;
	}

	evas_object_hide(pCtrlData->pLayout);
}


void MpVideoVolumeCtrlSetVolume(int nVol)
{
	VideoLogInfo("");

	if(pVolumeData == NULL)
	{
		VideoLogError("Volume Ctrl data is NULL");
		return;
	}
	MpVolumeCtrlData *pCtrlData = pVolumeData;

	if(nVol > pCtrlData->nMaxVol)
	{
		nVol = pCtrlData->nMaxVol;
	}

	if(nVol < 0)
	{
		nVol = 0;
	}

	MpVideoVolumeCtrlShow();

	pCtrlData->nCurVol = nVol;
	MpVideoVolumeCtrlSetCurVolume(nVol);
	MpVideoVolumeCtrlUpdateTextValue((void *)pCtrlData);
	MpVideoVolumeCtrlUpdateDragValue((void *)pCtrlData);

}

int MpVideoVolumeCtrlGetVolume()
{
	VideoLogInfo("");

	return MpVideoVolumeCtrlGetCurVolume();
}

bool MpVideoVolumeCtrlGetShowStatus()
{
	VideoLogInfo("");

	if(pVolumeData == NULL)
	{
		VideoLogError("Volume Ctrl data is NULL");
		return FALSE;
	}
	MpVolumeCtrlData *pCtrlData = pVolumeData;

	if(pCtrlData->pHideTimer)
	{
		return TRUE;
	}

	return FALSE;
}

