/*
 * Copyright (c) [2012] Samsung Electronics Co., Ltd.
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <app.h>
#include <glib.h>
#include <pmapi.h>
#include "mp-util.h"
#include "mp-video-log.h"
#include "video-player.h"
#include "mp-video-app-ctrl.h"
#include "mp-util-media-service.h"
#include "mp-video-string-define.h"
#include "mp-external-ug.h"
#include "mp-video-service-ctrl.h"

static void appResume(void *pUserData);

static bool bIsForeground = FALSE;

static Ecore_Idler *pInitIdlerHandle = NULL;

static Eina_Bool _mp_focus_in_cb(void *pUserData, int nType, void *pEvent)
{
	if (!pUserData) {
		VideoLogInfo("[ERR] No exist pUserData.");
		return ECORE_CALLBACK_PASS_ON;
	}

	if (!pEvent) {
		VideoLogInfo("[ERR] No exist pEvent.");
		return ECORE_CALLBACK_PASS_ON;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	Ecore_X_Event_Window_Focus_In *ev = (Ecore_X_Event_Window_Focus_In *)pEvent;

	if (ev->win == MpUtilGetMainWindowXID())
	{
		if(bIsForeground && MpVideoAppCtrlIsTopView(pUserData))
		{
			VideoLogInfo("resume for focus in");
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlResume(pAppData);
		}
	}

	return ECORE_CALLBACK_PASS_ON;
}

static Eina_Bool _mp_focus_out_cb(void *pUserData, int nType, void *pEvent)
{
	if (!pUserData) {
		VideoLogInfo("[ERR] No exist pUserData.");
		return ECORE_CALLBACK_PASS_ON;
	}

	if (!pEvent) {
		VideoLogInfo("[ERR] No exist pEvent.");
		return ECORE_CALLBACK_PASS_ON;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	Ecore_X_Event_Window_Focus_Out *ev = (Ecore_X_Event_Window_Focus_Out *)pEvent;

	if (ev->win == MpUtilGetMainWindowXID())
	{
		if(bIsForeground && MpVideoAppCtrlIsTopView(pUserData))
		{
			VideoLogInfo("pause for focus in");
			pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause(pAppData);
		}
	}

	return ECORE_CALLBACK_PASS_ON;

}


static Eina_Bool _mp_client_message_cb(void *pUserData, int nType, void *pEvent)
{
	if (!pUserData) {
		VideoLogInfo("[ERR] No exist pUserData.");
		return ECORE_CALLBACK_PASS_ON;
	}

	if (!pEvent) {
		VideoLogInfo("[ERR] No exist pEvent.");
		return ECORE_CALLBACK_PASS_ON;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	Ecore_X_Event_Client_Message *ev = (Ecore_X_Event_Client_Message *) pEvent;

	if (ev->message_type == ECORE_X_ATOM_E_ILLUME_QUICKPANEL_STATE)
	{
		if (ev->data.l[0] == ECORE_X_ATOM_E_ILLUME_QUICKPANEL_ON)
		{
			VideoLogInfo("quickpanel show");
			if(MpVideoAppCtrlIsTopView(pUserData) && bIsForeground)
			{
				pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlPause(pAppData);
			}
		}
		else
		{
			VideoLogInfo("quickpanel hide");
			if(MpVideoAppCtrlIsTopView(pUserData) && bIsForeground)
			{
				pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlResume(pAppData);
			}
		}
	}

	return ECORE_CALLBACK_PASS_ON;
}

static Eina_Bool _mp_app_init_idler_cb(void *pUserData)
{
	if (!pUserData) {
		VideoLogInfo("[ERR] No exist pUserData.");
		return ECORE_CALLBACK_CANCEL;
	}
	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	ecore_event_handler_add(ECORE_X_EVENT_CLIENT_MESSAGE, _mp_client_message_cb, (void *)pAppData);
	ecore_event_handler_add(ECORE_X_EVENT_WINDOW_FOCUS_IN, _mp_focus_in_cb, (void *)pAppData);
	ecore_event_handler_add(ECORE_X_EVENT_WINDOW_FOCUS_OUT, _mp_focus_out_cb, (void *)pAppData);

	pInitIdlerHandle = NULL;

	return ECORE_CALLBACK_CANCEL;

}

static void _mp_alloc_service_value(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR] No exist pUserData.");
		return;
	}


	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->szSubtitle = calloc(1, sizeof(char) * STR_LEN_MAX);
	pAppData->szVideoTitle = calloc(1, sizeof(char) * STR_LEN_MAX);

	pAppData->szMediaUri = calloc(1, sizeof(char) * STR_LEN_MAX);
	pAppData->szPreMediaUri = calloc(1, sizeof(char) * STR_LEN_MAX);
	pAppData->szNextMediaUri = calloc(1, sizeof(char) * STR_LEN_MAX);

	pAppData->szProxyAddress = calloc(1, sizeof(char) * STR_LEN_MAX);
	pAppData->szUserAgent = calloc(1, sizeof(char) * STR_LEN_MAX);
	pAppData->szCookie = calloc(1, sizeof(char) * STR_LEN_MAX);
}


static void _mp_free_service_value(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR] No exist pUserData.");
		return;
	}


	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(pAppData->szSubtitle)
	{
		free(pAppData->szSubtitle);
		pAppData->szSubtitle = NULL;
	}

	if(pAppData->szVideoTitle)
	{
		free(pAppData->szVideoTitle);
		pAppData->szVideoTitle = NULL;
	}

	if(pAppData->szMediaUri)
	{
		free(pAppData->szMediaUri);
		pAppData->szMediaUri = NULL;
	}
	if(pAppData->szPreMediaUri)
	{
		free(pAppData->szPreMediaUri);
		pAppData->szPreMediaUri = NULL;
	}

	if(pAppData->szNextMediaUri)
	{
		free(pAppData->szNextMediaUri);
		pAppData->szNextMediaUri = NULL;
	}

	if(pAppData->szProxyAddress)
	{
		free(pAppData->szProxyAddress);
		pAppData->szProxyAddress = NULL;
	}

	if(pAppData->szUserAgent)
	{
		free(pAppData->szUserAgent);
		pAppData->szUserAgent = NULL;
	}

	if(pAppData->szCookie)
	{
		free(pAppData->szCookie);
		pAppData->szCookie = NULL;
	}
}

static bool appCreate(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR] No exist pUserData.");
		return FALSE;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	_mp_alloc_service_value((void *)pAppData);

	pInitIdlerHandle = ecore_idler_add(_mp_app_init_idler_cb, (void *)pAppData);

#ifdef TARGET
	elm_config_preferred_engine_set("opengl_x11");
#else
	elm_config_preferred_engine_set("software_x11");
#endif

	elm_theme_extension_add(NULL, VIDEO_CUSTOM_THEME);


	MpUtilInitSession();
	MpUtilVolumeTypeSet();

	if(!MpUtilMMCstatusNoti(pAppData))
	{
		VideoLogError("[ERR] Fail to set MMC notification.");
	}

	bIsForeground = FALSE;

	MpVideoAppCtrlSetForCheckingForegroundwindow(FALSE);

	return TRUE;
}

static void appService(service_h pAppSvcHandle, void *pUserData)
{
	VideoLogInfo("");

	if(!pUserData)
	{
		VideoLogInfo("[ERR] No exist pUserData.");
		bIsForeground = FALSE;
		MpVideoAppCtrlSetForCheckingForegroundwindow(FALSE);
		return;
	}

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	char* szGetMediaUri = NULL;
	MpUtilVolumeTypeSet();

	if(!pAppSvcHandle)
	{
		VideoLogInfo("[ERR] No exist pBundle.");
		bIsForeground = FALSE;
		MpVideoAppCtrlSetForCheckingForegroundwindow(FALSE);
		return;
	}

	if(pAppData->pMainWindow == NULL)
	{
		VideoLogInfo("Create main window.");
		pAppData->pMainWindow = (Evas_Object *)MpUtilCreateMainWindow(PACKAGE);
	}
	else
	{
		if(strlen(pAppData->szMediaUri) > 0)
		{
			MpVideoAppCtrlResetForRelaunching(pAppData);
		}
	}

 	if(MpVideoServiceCtrlInitServiceParser((void*)pAppSvcHandle, &szGetMediaUri) == MP_ERROR_PLAYER_TYPE)
	{
		VideoLogInfo("[ERR] Fail to init service parser.");
		bIsForeground = FALSE;
		MpVideoAppCtrlSetForCheckingForegroundwindow(FALSE);
		return;
	}

	if(szGetMediaUri)
	{
		strncpy(pAppData->szMediaUri, szGetMediaUri, STR_LEN_MAX);
		VideoLogInfo("szGetMediaUri : %s", pAppData->szMediaUri);
		free(szGetMediaUri);
	}
	else
	{
		VideoLogInfo("No exist szGetMediaUri");
	}

	if(!MpUtilBatteryStatusNoti(pAppData))
	{
		VideoLogInfo("[ERR] Fail to set Battery notification.");
		bIsForeground = FALSE;
		MpVideoAppCtrlSetForCheckingForegroundwindow(FALSE);
		return;
	}
	bIsForeground = TRUE;

	MpUtilMediaSvcInitSession();
	MpUtilSetKeyGrab(HOME_KEY, KEY_GRAB_SHARED);
	MpVideoAppCtrlInit(pAppData);
}

static void appExit(void *pUserData)
{
	VideoLogInfo("");

	if (!pUserData) {
		VideoLogInfo("[ERR] No exist pUserData.");
		return;
	}
	VideoAppData *pAppData = (VideoAppData *)pUserData;

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);

	MpUtilMultimediaVolumeKeyUngrab();
	MpUtilLcdOffForPowerButton();
	MpUtilSetKeyUngrab(HOME_KEY);
	MpUtilMultimediaKeyUngrab();
	MpUtilMMCstatusNotiIgnore();
	MpUtilBatteryStatusNotiIgnore();
	MpUtilVolumeTypeUnset();
	MpUtilFinishSession();
	MpUtilDeletePopupHandle();
	MpUtilMediaSvcDestoryVideoList();
	MpUtilMediaSvcFinishSession();

	_mp_free_service_value(pUserData);

	bIsForeground = FALSE;
	MpVideoAppCtrlSetForCheckingForegroundwindow(FALSE);

#ifdef TA_SVAE_PERF_FILE
	VP_PERF_DEINIT();
#endif
}

static void appPause(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist pUserdata handle.");
		return;
	}

	VideoLogInfo("");

	MpUtilVolumeTypeUnset();
	MpUtilMultimediaVolumeKeyUngrab();

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(MpVideoAppCtrlIsCheckCallStatus())
	{
		VideoLogInfo("Skip for Call Pause.");
		MpVideoAppCtrlInitCheckCallStatus();
	}

	int nTmpPlayerType = MpVideoServiceCtrlGetPlayerType();

	if(nTmpPlayerType == MP_VIDEO_PLAYER ||
			nTmpPlayerType == MP_VIDEO_PLAYER_SIMPLE ||
			nTmpPlayerType == MP_VIDEO_PLAYER_GALLERY ||
			nTmpPlayerType == MP_VIDEO_PLAYER_EMAIL ||
			nTmpPlayerType == MP_VIDEO_PLAYER_MMS)
	{
		VideoLogInfo("Pause application at video player.");
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}
	else if(nTmpPlayerType == MP_STREAMING_PLAYER)
	{
		VideoLogInfo("Pause application at streaming player.");
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlExit(pAppData);
	}

	pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlHideLayout(pAppData);

	MpUtilDeletePopupHandle();

	elm_win_indicator_mode_set(pAppData->pMainWindow, ELM_WIN_INDICATOR_SHOW);

	bIsForeground = FALSE;
	MpVideoAppCtrlSetForCheckingForegroundwindow(FALSE);

	evas_object_hide(pAppData->pMainWindow);
}

static void appResume(void *pUserData)
{
	if(!pUserData)
	{
		VideoLogInfo("No exist pUserdata handle.");
		return;
	}

	VideoLogInfo("");

	VideoAppData *pAppData = (VideoAppData *)pUserData;

	if(bIsForeground)
	{
		VideoLogInfo("Defence overlap launching.");
		return;
	}
	MpUtilVolumeTypeSet();

	MpUtilMultimediaVolumeKeyGrab();

	if(!MpVideoAppCtrlIsTopView(pUserData))
	{
		VideoLogError("Overlay is not staied in top.");
		return;
	}

	if(MpVideoAppCtrlIsExistPlayerHandle())
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlResume(pAppData);
	}
	else
	{
		pAppData->VideoAppCtrlCallbackFunc.VideoAppCtrlStartByAppResume(pAppData);
	}

	elm_win_indicator_mode_set(pAppData->pMainWindow, ELM_WIN_INDICATOR_SHOW);

	bIsForeground = TRUE;
	MpVideoAppCtrlSetForCheckingForegroundwindow(TRUE);

	evas_object_show(pAppData->pMainWindow);
}

static void appRotate(app_device_orientation_e rotateState, void *pUserData)
{
	VideoLogInfo("");

	int nCurRotate = 0;

	switch(rotateState)
	{
	case APP_DEVICE_ORIENTATION_0:
		VideoLogInfo("VIDEO_ROTATE_PORTRAIT_NORMAL");
		nCurRotate = VIDEO_ROTATE_PORTRAIT_NORMAL;
		break;

	case APP_DEVICE_ORIENTATION_90:
		VideoLogInfo("VIDEO_ROTATE_LANDSCAPE_REVERSE");
		nCurRotate = VIDEO_ROTATE_LANDSCAPE_REVERSE;
		break;

	case APP_DEVICE_ORIENTATION_180:
		VideoLogInfo("VIDEO_ROTATE_PORTRAIT_REVERSE");
		nCurRotate = VIDEO_ROTATE_PORTRAIT_REVERSE;
		break;

	case APP_DEVICE_ORIENTATION_270:
		VideoLogInfo("VIDEO_ROTATE_LANDSCAPE_NORMAL");
		nCurRotate = VIDEO_ROTATE_LANDSCAPE_NORMAL;
		break;
	}

	MpVideoAppCtrlSetRotation(nCurRotate, pUserData);
}

int main(int argc, char *argv[])
{
	VideoAppData pAppData;
	memset(&pAppData, 0, sizeof(VideoAppData));

	app_event_callback_s st_appEventService;

	st_appEventService.create = appCreate;
	st_appEventService.terminate = appExit;
	st_appEventService.pause = appPause;
	st_appEventService.resume = appResume;
	st_appEventService.service = appService;
	st_appEventService.low_memory = NULL;
	st_appEventService.low_battery = NULL;
	st_appEventService.device_orientation = appRotate;
	st_appEventService.language_changed = NULL;
	st_appEventService.region_format_changed = NULL;

	if(app_efl_main(&argc, &argv, &st_appEventService, &pAppData) != APP_ERROR_NONE)
	{
		VideoLogInfo("[ERR] app_efl_main().");
		return -1;
	}

	VideoLogInfo("escape app_efl_main() loop.");

	return 0;
}
